<?php
/*!*******************************************************************************
*! FILE NAME  : snmp_demo.php
*! DESCRIPTION: Demo script to use net-snmp extensions  
*! Copyright (C) 2008 Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!  $Log: snmp_demo.php,v $
*!  Revision 1.1  2008/12/25 18:45:14  landy_
*!  Add dropbear and snmp agent to flash image
*!
*!  Revision 1.1  2008/11/07 07:49:30  landy_
*!  include SNMP support to camera flashimage
*!
*!
*!
 */


define("REGEXP_SNMP_TRIM", "(hex|counter(32|64)|gauge|gauge(32|64)|float|ipaddress|string|integer):");
define("SNMP_METHOD_PHP", 1);
define("SNMP_METHOD_BINARY", 2);

define("SNMP_ESCAPE_CHARACTER", "'");

function snmp_get($hostname , $community , 
		$oid, $version, $username ="", $password = "", 
		$auth_proto = "", $priv_pass = "", $priv_proto = "", $context ="", 
		$port = 161, $timeout = 500, $retries = 0 ) {

	/* determine default retries */
	if (($retries == 0) || (!is_numeric($retries))) 
		$retries = 3;

	/* do not attempt to poll invalid combinations */
	if (($version == 0) || (($community == "") && ($version != 3))) 
		return "U";

	if (snmp_get_method($version) == SNMP_METHOD_PHP) {
	/* make sure snmp* is verbose so we can see what types of data 	we are getting back */
	#	snmp_set_quick_print(0);

		if ($version == "1") {
			$snmp_value = @snmpget("$hostname:$port", "$community", "$oid", 
				($timeout * 1000), $retries);
		}elseif ($version == "2") {
			$snmp_value = @snmp2_get("$hostname:$port", "$community", "$oid", 
						 ($timeout * 1000), $retries);
		}else{
			if ($priv_proto == "[None]") {
				$proto = "authNoPriv";
				$priv_proto = "";
			}else{
				$proto = "authPriv";
			}
			$snmp_value = @snmp3_get("$hostname:$port", "$username", $proto, 
						  $auth_proto, "$password", $priv_proto, 
						  "$priv_pass", "$oid", ($timeout * 1000), 
						  $retries);
		}
	}else {
		/* we not compile utility, we use extension of PHP */
		return "U";
	}

	if (isset($snmp_value)) {
		/* fix for multi-line snmp output */
		if (is_array($snmp_value)) 
			$snmp_value = implode(" ", $snmp_value);
	}

	/* strip out non-snmp data */
	$snmp_value = format_snmp_string($snmp_value);

	return $snmp_value;
}

function snmp_getnext($hostname, $community, $oid, $version, $username, 
	$password, $auth_proto, $priv_pass, $priv_proto, $context, 
	$port = 161, $timeout = 500, $retries = 0) {

	/* determine default retries */
	if (($retries == 0) || (!is_numeric($retries))) 
		$retries = 3;

	/* do not attempt to poll invalid combinations */
	if (($version == 0) || (($community == "") && ($version != 3))) 
		return "U";

	if (snmp_get_method($version) == SNMP_METHOD_PHP) {
	/* make sure snmp* is verbose so we can see what types of data 	we are getting back */
		snmp_set_quick_print(0);

		if ($version == "1") {
			$snmp_value = @snmpgetnext("$hostname:$port", "$community", 
						   "$oid", ($timeout * 1000), $retries);
		}elseif ($version == "2") {
			$snmp_value = @snmp2_getnext("$hostname:$port", "$community", 
						     "$oid", ($timeout * 1000), $retries);
		}else{
			if ($priv_proto == "[None]") {
				$proto = "authNoPriv";
				$priv_proto = "";
			}else{
				$proto = "authPriv";
			}

			$snmp_value = @snmp3_getnext("$hostname:$port", "$username", $proto, $auth_proto, 
						     "$password", $priv_proto, "$priv_pass", 
						     "$oid", ($timeout * 1000), $retries);
		}
	}else {
		/* we not compile utility, we use extension of PHP */
		return "U";
	}
	if (isset($snmp_value)) {
		/* fix for multi-line snmp output */
		if (is_array($snmp_value)) {
			$snmp_value = implode(" ", $snmp_value);
		}
	}

	/* strip out non-snmp data */
	$snmp_value = format_snmp_string($snmp_value);

	return $snmp_value;
}


function format_snmp_string($string) {
	/* strip off all leading junk (the oid and stuff) */
	$string = trim(ereg_replace(".*= ?", "", $string));

	/* remove ALL quotes */
	$string = str_replace("\"", "", $string);
	$string = str_replace("'", "", $string);
	$string = str_replace(">", "", $string);
	$string = str_replace("<", "", $string);
	$string = str_replace("\\", "", $string);

	/* Account for invalid type messages */
	if (substr_count($string, "Wrong Type")) {
		$string = strrev($string);
		if ($position = strpos($string, ":")) {
			$string = trim(strrev(substr($string, 0, $position)));
		}else{
			$string = trim(strrev($string));
		}
	}

	/* Remove invalid chars */
#	$k = strlen($string);
#	for ($i=0; $i < $k; $i++) {
#		if ((ord($string[$i]) <= 31) || (ord($string[$i]) >= 127)) {
#			$string[$i] = " ";
#		}
#	}
#	$string = trim($string);

	if ((substr_count($string, "Hex-STRING:")) ||
		(substr_count($string, "Hex:"))) {
		/* strip of the 'Hex-STRING:' */
		$string = eregi_replace("Hex-STRING: ?", "", $string);
		$string = eregi_replace("Hex: ?", "", $string);

		$string_array = split(" ", $string);

		/* loop through each string character and make ascii */
		$string = "";
		$hexval = "";
		$ishex  = false;
		for ($i=0;($i<sizeof($string_array));$i++) {
			if (strlen($string_array[$i])) {
				$string .= chr(hexdec($string_array[$i]));

				$hexval .= str_pad($string_array[$i], 2, "0", STR_PAD_LEFT);

				if (($i+1) < count($string_array)) {
					$hexval .= ":";
				}

				if ((hexdec($string_array[$i]) <= 31) || (hexdec($string_array[$i]) >= 127)) {
					if ((($i+1) == sizeof($string_array)) && ($string_array[$i] == 0)) {
						/* do nothing */
					}else{
						$ishex = true;
					}
				}
			}
		}

		if ($ishex) $string = $hexval;
	}elseif (preg_match("/(hex:\?)?([a-fA-F0-9]{1,2}(:|\s)){5}/", $string)) {
		$octet = "";

		/* strip of the 'hex:' */
		$string = eregi_replace("hex: ?", "", $string);

		/* split the hex on the delimiter */
		$octets = preg_split("/\s|:/", $string);

		/* loop through each octet and format it accordingly */
		for ($i=0;($i<count($octets));$i++) {
			$octet .= str_pad($octets[$i], 2, "0", STR_PAD_LEFT);

			if (($i+1) < count($octets)) {
				$octet .= ":";
			}
		}

		/* copy the final result and make it upper case */
		$string = strtoupper($octet);
	}elseif (preg_match("/Timeticks:\s\((\d+)\)\s/", $string, $matches)) {
		$string = $matches[1];
	}

	$string = eregi_replace(REGEXP_SNMP_TRIM, "", $string);

	return trim($string);
}

function snmp_escape_string($string) {
	if (substr_count($string, SNMP_ESCAPE_CHARACTER)) {
		$string = substr_replace(SNMP_ESCAPE_CHARACTER, "\\" . SNMP_ESCAPE_CHARACTER, $string);
	}

	return SNMP_ESCAPE_CHARACTER . $string . SNMP_ESCAPE_CHARACTER;
}

function snmp_get_method($version = 1) {
	if ((function_exists("snmpget")) && ($version == 1)) {
		return SNMP_METHOD_PHP;
	}else if ((function_exists("snmp2_get")) && ($version == 2)) {
		return SNMP_METHOD_PHP;
	}else if ((function_exists("snmp3_get")) && ($version == 3)) {
		return SNMP_METHOD_PHP;
	}else{
		/* looks like snmp is broken */
		return SNMP_METHOD_BINARY;
	}
}



if (!extension_loaded("snmp")) { 
	print "No SNMP extention in PHP\n";
	return;
}
else print "SNMP extention in PHP are ON!\n<br>";

$res = snmp_get("localhost" , "E353" ,"sysDescr.0", 2); 
 $string = preg_replace("/\n/","\n<br>", $res);
 print "System description: ".$string."\n<br>";
 
 $res = snmp_get("localhost" , "E353" ,"sysLocation.0", 2); 
 $string = preg_replace("/\n/","\n<br>", $res);
 print "System location: ".$string."\n<br>";
 
  
 $res = snmp_get("localhost" , "E353" ,"ifPhysAddress.2", 2);
 $string = preg_replace("/\n/","\n<br>", $res);
 print "\n<br>Serial #: ".$string."\n<br>";
 
 $res = snmp_get("localhost" , "E353" ,"sysUpTimeInstance", 2);
 $string = preg_replace("/\n/","\n<br>", $res);
 print "\n<br>System(Camera) uptime: ".$string."\n<br>";

 print "<br>Extending net-snmp (look at /etc/snmpd.conf file)</br>"; 
 
 print "<br>Running processes:<br>";
 $res = snmp_get("localhost" , "E353" ,".1.3.6.1.4.1.2021.51.3.1.2.2.112.115", 1); 
 $string = preg_replace("/\n/","\n<br>", $res);
 print $string."\n<br>";
 
 print "<br>Disk usage:<br>";
 $res = snmp_get("localhost" , "E353" ,"ucdavis.50.3.1.2.2.100.102", 1); 
 $string = preg_replace("/\n/","\n<br>", $res);
 print $string."\n<br>";

# print "<br>Memory free:<br>";
# $res = snmp_get("localhost" , "E353" ,"ucdavis.52.3.1.2.4.102.114.101.101", 1); 
# $string = preg_replace("/\n/","\n<br>", $res);
# print $string."\n<br>";



?>
