/*
** -----------------------------------------------------------------------------**
** pixproc.v
**
** Combines two CCD outputs in a single stream
**
** Copyright (C) 2003-2007 Elphel, Inc.
**
** -----------------------------------------------------------------------------**
**  This file is part of x347
**  x347 is free software - hardware description language (HDL) code.
** 
**  This program is free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>.
** -----------------------------------------------------------------------------**
**
**  $Log: merge_streams.v,v $
**  Revision 1.1.1.1  2008/11/27 20:04:02  elphel
**
**
**  Revision 1.1.1.1  2007/08/17 12:07:19  elphel
**  This is a fresh tree based on elphel353-2.10
**
**  Revision 1.6  2007/08/17 12:07:19  spectr_rain
**  switch to GPL3 license
**
**  Revision 1.5  2007/07/20 09:46:13  spectr_rain
**  *** empty log message ***
**
**  Revision 1.5  2007/05/25 02:09:09  elphel
**  347 update for mechanical shutter
**
**  Revision 1.1  2006/09/24 16:44:36  elphel
**  Split file into separate module files, added real time clock module, project files
**
*/
module   merge_streams (clk,   // ADC clock, 25MHz
                        clk2x, // output clock, twice clk. 0->1 earlier than that of clk
                        en,    // enabels operation (==frame)
                        line,  // only leading edge is used
                        ll,    // [10:0] - half line length.10 MSBs define (always even) line length to process, LSB - delay start by 1 pixel (to match 2 halves)
                        da,    // [13:0] channel A data in
                        db,    // [13:0] channel B data in
                        do,    // [13:0] channel A data out (nothing to channel b)
                        line_out); // output line data valid

  input         clk, clk2x, en, line;
  input  [10:0] ll;
  input  [13:0] da;
  input  [13:0] db;
  output [13:0] do;
  output        line_out;

  wire   [15:0] qa; // RAM output channel1 (2 MSBs not used)
//  wire   [13:0] qb; // RAM output channel2
  wire   [15:0] qb; // RAM output channel2 (2 MSBs not used)
  reg           we,re,re_a, re_b, sel_b;
//  wire   [ 9:0] wa_a;
//  wire   [10:0] wa_b;
//  wire   [ 9:0] ra_a;
//  wire   [10:0] ra_b;

  reg           line_d;
  wire          line_start;
  wire          start_out0;   //2 pixels delayed from the half, sync to clk
  reg    [ 1:0] start_out_s;
  reg           start_out;    // sync to clk2x, single-cycle start of line out
  reg    [10:0]  ipixcnt;  // counts input pixels (down), MSB - half
// for channel A output takes place during the second half of input, needed buffer - 1024 pixels.
// for channel B output overlaps with the next line (during first half if small gap), in reversed order.
// So the buffer needed is full 2048 (half line), and the order is alternating with each line.
// on even lines input address goes from 0 to max, on odd - from 2047  to 2047-max.
// to use the same addresses channel A can use the same as B, alternating lines

  reg    [11:0]  opixcnt;  // counts input pixels (down), MSB - half


  reg           i_odd, o_odd, o_dir; // input line odd, output line odd (needed for channel b buffer management)
  reg    [10:0] wa; // pixel counter for write address
  reg    [10:0] ra; // pixel counter for read address


  assign        do[13:0]=sel_b?qb[13:0]:qa[13:0]; // use register if needed

  always @ (posedge clk) line_d <=line;
  SRL16 i_line_start (.Q(line_start),  .A0(ll[0]), .A1(1'b1), .A2(1'b0), .A3(1'b1), .CLK(clk), .D(line && !line_d)); // add extra to fit into 4096 pixels
  always @ (posedge clk) begin
    if (!we || (ipixcnt[9:0]==10'b0)) ipixcnt[9:0]<= ll[10:1];
    else ipixcnt[9:0] <= ipixcnt[9:0]-1;
    ipixcnt[10] <= we && (ipixcnt[10] || (ipixcnt[9:0]==10'b0));
    we <= en && (line_start || (we && (ipixcnt[10:0]!=11'h400)));
    i_odd <= !en || (line_start ^ i_odd);
    if (!en || line_start) wa[10:0] <= {11{!i_odd}};
    else if (we)  wa[10:0] <= i_odd? (wa[10:0]-1): (wa[10:0]+1); 

  end


  SRL16 i_start_out0 (.Q(start_out0),  .A0(1'b1), .A1(1'b0), .A2(1'b0), .A3(1'b0), .CLK(clk), .D(we && (ipixcnt[10:0]==11'h0))); //
  always @ (posedge clk2x) begin
    start_out_s[1:0] <={start_out_s[0],start_out0};
    start_out        <= start_out_s[0] && ! start_out_s[1];

    if (!re || (opixcnt[10:0]==11'b0)) opixcnt[10:0]<= {ll[10:1],1'b1};
    else opixcnt[10:0] <= opixcnt[10:0]-1;
    opixcnt[11] <= re && (opixcnt[11] || (opixcnt[10:0]==11'b0));
    if (start_out || (opixcnt[10:0]==11'b0)) o_dir <= !o_odd;

    re   <= en && (start_out || (re   && (opixcnt[11:0]!=12'h800)));
//That was for in 324    
/*
    re_a <= en && (start_out || (re_a && (opixcnt[10:0]!=11'b0  )));
    re_b <= en && ((opixcnt[10:0]==11'b0)?re_a:re_b );
    sel_b<= re_b;
*/    
    re_b <= en && (start_out || (re_b && (opixcnt[10:0]!=11'b0  )));
    re_a <= en && ((opixcnt[10:0]==11'b0)?re_b:re_a );
    sel_b<= ~re_a;

    o_odd <= !en || (start_out?i_odd:o_odd);

    if (!en || start_out) ra[10:0] <= {11{!o_odd}};
    else if (re && (opixcnt[10:0]!=11'b0))  ra[10:0] <= o_dir? (ra[10:0]-1): (ra[10:0]+1); 
  end

  SRL16 i_line_out (.Q(line_out), .A0(1'b0), .A1(1'b0), .A2(1'b0), .A3(1'b0), .CLK(clk2x), .D(re));   //change if data registered

/*
  // will probably remove wires later...
  assign wa_a[ 9:0] = wa[ 9:0];
  assign wa_b[10:0] = wa[10:0];
  assign ra_a[ 9:0] = ra[ 9:0];
  assign ra_b[10:0] = ra[10:0];


  RAMB16_S18_S18 i_qa1500 (.ADDRA(wa_a[ 9:0]),.DIA({2'b0,da[13: 0]}),.DOA(),     .CLKA(clk),  .ENA(we  ),.SSRA(1'b0),.WEA(1'b1),
                           .ADDRB(ra_a[ 9:0]),.DOB(qa[15: 0]),.DIB(16'b0),.CLKB(clk2x),.ENB(re_a),.SSRB(1'b0),.WEB(1'b0),
                           .DIPA(2'h0),.DOPA(),.DIPB(2'h0),.DOPB());

  RAMB16_S9_S9 i_qb1508(.ADDRA(wa_b[10:0]),.DIA({2'b0,db[13: 8]}),.DOA(), .CLKA(clk),  .ENA(we  ),.SSRA(1'b0),.WEA(1'b1),
                        .ADDRB(ra_b[10:0]),.DOB(qb[15: 8]), .DIB(8'b0),   .CLKB(clk2x),.ENB(re_b),.SSRB(1'b0),.WEB(1'b0),
                        .DIPA (1'h0),      .DOPA(),         .DIPB(1'h0),  .DOPB());
  RAMB16_S9_S9 i_qb0700(.ADDRA(wa_b[10:0]),.DIA(db[ 7: 0]), .DOA(),       .CLKA(clk),  .ENA(we  ),.SSRA(1'b0),.WEA(1'b1),
                        .ADDRB(ra_b[10:0]),.DOB(qb[ 7: 0]), .DIB(8'b0),   .CLKB(clk2x),.ENB(re_b),.SSRB(1'b0),.WEB(1'b0),
                        .DIPA (1'h0),      .DOPA(),         .DIPB(1'h0),  .DOPB());

  RAMB16_S18_S18 i_qa1500 (.ADDRA(wa[ 9:0]),.DIA({2'b0,da[13: 0]}),.DOA(),     .CLKA(clk),  .ENA(we  ),.SSRA(1'b0),.WEA(1'b1),
                           .ADDRB(ra[ 9:0]),.DOB(qa[15: 0]),.DIB(16'b0),.CLKB(clk2x),.ENB(re_a),.SSRB(1'b0),.WEB(1'b0),
                           .DIPA(2'h0),.DOPA(),.DIPB(2'h0),.DOPB());

  RAMB16_S9_S9 i_qb1508(.ADDRA(wa[10:0]),.DIA({2'b0,db[13: 8]}),.DOA(), .CLKA(clk),  .ENA(we  ),.SSRA(1'b0),.WEA(1'b1),
                        .ADDRB(ra[10:0]),.DOB(qb[15: 8]), .DIB(8'b0),   .CLKB(clk2x),.ENB(re_b),.SSRB(1'b0),.WEB(1'b0),
                        .DIPA (1'h0),    .DOPA(),         .DIPB(1'h0),  .DOPB());
  RAMB16_S9_S9 i_qb0700(.ADDRA(wa[10:0]),.DIA(db[ 7: 0]), .DOA(),       .CLKA(clk),  .ENA(we  ),.SSRA(1'b0),.WEA(1'b1),
                        .ADDRB(ra[10:0]),.DOB(qb[ 7: 0]), .DIB(8'b0),   .CLKB(clk2x),.ENB(re_b),.SSRB(1'b0),.WEB(1'b0),
                        .DIPA (1'h0),    .DOPA(),         .DIPB(1'h0),  .DOPB());


*/
  RAMB16_S18_S18 i_qb1500 (.ADDRA(wa[ 9:0]),.DIA({2'b0,db[13: 0]}),.DOA(),     .CLKA(clk),  .ENA(we  ),.SSRA(1'b0),.WEA(1'b1),
                           .ADDRB(ra[ 9:0]),.DOB(qb[15: 0]),.DIB(16'b0),.CLKB(clk2x),.ENB(re_b),.SSRB(1'b0),.WEB(1'b0),
                           .DIPA(2'h0),.DOPA(),.DIPB(2'h0),.DOPB());

  RAMB16_S9_S9 i_qa1508(.ADDRA(wa[10:0]),.DIA({2'b0,da[13: 8]}),.DOA(), .CLKA(clk),  .ENA(we  ),.SSRA(1'b0),.WEA(1'b1),
                        .ADDRB(ra[10:0]),.DOB(qa[15: 8]), .DIB(8'b0),   .CLKB(clk2x),.ENB(re_a),.SSRB(1'b0),.WEB(1'b0),
                        .DIPA (1'h0),    .DOPA(),         .DIPB(1'h0),  .DOPB());
  RAMB16_S9_S9 i_qa0700(.ADDRA(wa[10:0]),.DIA(da[ 7: 0]), .DOA(),       .CLKA(clk),  .ENA(we  ),.SSRA(1'b0),.WEA(1'b1),
                        .ADDRB(ra[10:0]),.DOB(qa[ 7: 0]), .DIB(8'b0),   .CLKB(clk2x),.ENB(re_a),.SSRB(1'b0),.WEB(1'b0),
                        .DIPA (1'h0),    .DOPA(),         .DIPB(1'h0),  .DOPB());



 endmodule


