/*
** -----------------------------------------------------------------------------**
** small_modules.v
**
** Small low-level modules
**
** Copyright (C) 2003-2006 Elphel, Inc.
**
** -----------------------------------------------------------------------------**
**  This file is part of x324
**  x324 is free software - hardware description language (HDL) code.
** 
**  This program is free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>.
** -----------------------------------------------------------------------------**
**
**  $Log: small_modules.v,v $
**  Revision 1.1.1.1  2008/11/27 20:04:02  elphel
**
**
**  Revision 1.1.1.1  2007/08/17 12:07:19  elphel
**  This is a fresh tree based on elphel353-2.10
**
**  Revision 1.6  2007/08/17 12:07:19  spectr_rain
**  switch to GPL3 license
**
**  Revision 1.5  2007/07/20 09:46:13  spectr_rain
**  *** empty log message ***
**
**  Revision 1.4  2007/05/25 02:09:09  elphel
**  347 update for mechanical shutter
**
**  Revision 1.1  2006/09/24 16:44:36  elphel
**  Split file into separate module files, added real time clock module, project files
**
*/

module RAM16XnD(D,WE,CLK,AW,AR,QW,QR);
parameter NBits=16;
   input  [NBits-1:0] D;
   input              WE,CLK;
   input  [ 3:0]  AW;
   input  [ 3:0]  AR;
   output [NBits-1:0]   QW;
   output [NBits-1:0]   QR;
   reg    [NBits-1:0]   ram [0:15];
   always @ (posedge CLK) if (WE) ram[AW] <= D; 
   assign   QW= ram[AW];
   assign   QR= ram[AR];
endmodule


module sdo1(c,d,q);
    input c;
    input  d;
    output q;
 sdo0 i_q  (.c(c),.d(d),.q(q));
// synthesis attribute KEEP_HIERARCHY of i_q  is "TRUE"
endmodule

module sdo0(c,d,q); // similar to OFD, but the place to add speed
    input c;
    input d;
    output q;
wire dr;
OBUF_F_12 i_q (.I(dr), .O(q));
FD i_dr (.C(c), .D(d), .Q(dr));
//synthesis translate_off
 defparam i_dr.INIT = 1'b1;
//synthesis translate_on
//synthesis attribute INIT of i_dr is "1" 
// synthesis attribute IOB of i_dr is "TRUE"
endmodule


// similar pairs of modules - one for each of the fast signals to be able to adjust output impedance
// and so the delay

// RG
module out_rg(c,d,q);
    input c;
    input  d;
    output q;
 out_rg0 i_q  (.c(c),.d(d),.q(q));
// synthesis attribute KEEP_HIERARCHY of i_q  is "TRUE"
endmodule

module out_rg0(c,d,q); // similar to OFD, but the place to add speed
    input c;
    input d;
    output q;
wire dr;
OBUF_F_12 i_q (.I(dr), .O(q));
FD i_dr (.C(c), .D(d), .Q(dr));
//synthesis translate_off
 defparam i_dr.INIT = 1'b1;
//synthesis translate_on
//synthesis attribute INIT of i_dr is "1" 
// synthesis attribute IOB of i_dr is "TRUE"
endmodule

// PH1
module out_ph1(c,d,q);
    input c;
    input  d;
    output q;
 out_ph10   i_q  (.c(c),.d(d),.q(q));
// synthesis attribute KEEP_HIERARCHY of i_q  is "TRUE"
endmodule

module out_ph10(c,d,q); // similar to OFD, but the place to add speed
    input c;
    input d;
    output q;
wire dr;
OBUF_F_12 i_q (.I(dr), .O(q));
FD i_dr (.C(c), .D(d), .Q(dr));
//synthesis translate_off
 defparam i_dr.INIT = 1'b1;
//synthesis translate_on
//synthesis attribute INIT of i_dr is "1" 
// synthesis attribute IOB of i_dr is "TRUE"
endmodule

// PH1L
module out_ph1l(c,d,q);
    input c;
    input  d;
    output q;
 out_ph1l0  i_q  (.c(c),.d(d),.q(q));
// synthesis attribute KEEP_HIERARCHY of i_q  is "TRUE"
endmodule

module out_ph1l0(c,d,q); // similar to OFD, but the place to add speed
    input c;
    input d;
    output q;
wire dr;
OBUF_F_12 i_q (.I(dr), .O(q));
FD i_dr (.C(c), .D(d), .Q(dr));
//synthesis translate_off
 defparam i_dr.INIT = 1'b1;
//synthesis translate_on
//synthesis attribute INIT of i_dr is "1" 
// synthesis attribute IOB of i_dr is "TRUE"
endmodule

// PH2
module out_ph2(c,d,q);
    input c;
    input  d;
    output q;
 out_ph20   i_q  (.c(c),.d(d),.q(q));
// synthesis attribute KEEP_HIERARCHY of i_q  is "TRUE"
endmodule

module out_ph20(c,d,q); // similar to OFD, but the place to add speed
    input c;
    input d;
    output q;
wire dr;
OBUF_F_12 i_q (.I(dr), .O(q));
FD i_dr (.C(c), .D(d), .Q(dr));
//synthesis translate_off
 defparam i_dr.INIT = 1'b1;
//synthesis translate_on
//synthesis attribute INIT of i_dr is "1" 
// synthesis attribute IOB of i_dr is "TRUE"
endmodule


module ipadqff(c,e,q,qr,d);  //
    input c;
    input e;
    output q;
    output qr;
    input d;
   ipadqff0 i_q (.c(c),.e(e),.q(q),.qr(qr),.d(d));
// s-ynthesis attribute KEEP_HIERARCHY of i_q is "TRUE"
endmodule

module ipadqff0(c,e,q,qr,d);
    input c;
    input e;
    output q;
    output qr;
    input d;
  IBUF  i_q (.I(d), .O(q));
  FDE   i_qr (.C(c), .CE(e), .D(q), .Q(qr));
// s-ynthesis attribute IOB of i_qr is "TRUE"
endmodule
