/*
** -----------------------------------------------------------------------------**
** x347.v
**
** Top level module
**
** Copyright (C) 2007 Elphel, Inc.
**
** -----------------------------------------------------------------------------**
**  This file is part of x347
**  x347 is free software - hardware description language (HDL) code.
** 
**  This program is free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
** 
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
** 
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>.
** -----------------------------------------------------------------------------**
**
**  $Log: x347.v,v $
**  Revision 1.1.1.1  2008/11/27 20:04:02  elphel
**
**
**  Revision 1.2  2007/10/12 22:59:37  elphel
**  Changing control of powering secondary units (separate analog power and shutter, new interface)
**
**  Revision 1.1.1.1  2007/10/10 03:53:37  elphel
**  This is a fresh tree based on elphel353-2.10
**
**  Revision 1.8  2007/08/24 05:38:02  elphel
**  Bug fix - was wrong manual merging of x347.v, timing.v
**
**  Revision 1.7  2007/08/21 21:56:22  elphel
**  Changed the duty cycle of the pixel clock from the 10347 board to the 10353. Added possibility to adjust the phase of that clock in 4ns steps (currently 40 degrees)
**
**  Revision 1.6  2007/08/17 12:07:19  spectr_rain
**  switch to GPL3 license
**
**  Revision 1.5  2007/07/20 09:46:13  spectr_rain
**  *** empty log message ***
**
**  Revision 1.6  2007/05/25 02:09:09  elphel
**  347 update for mechanical shutter
**
**  Revision 1.3  2007/04/18 03:05:25  elphel
**  changing x347 to use larger BlockRAM (it was using Spartan 2e ones)
**
**
*/
/*module x324(CLK0,CLK1,CLK2,CLK3,DA,DB,FSA,FSB,DCLKA,DCLKB,ADA,ADB,4,SB0,SI0,SA1,SB1,
               SI1,SA2,SB2,SI2,SA3,SB3,SI3,DV,DVW1,DVW2,DVLD,DAC8C,DAC8D,
               DAC8FS,DAC8LD,DAC8R,SHTR,FD,CSRG,CSH1,CSH1L,CSH2,VHSCL,VHSDI,
               PH1,PH1L,PH2,RG,ENAPWR,GP,ADCCLK,SHPA,SHDA,SHPB,SHDB,CLPOB,
               CLPDM,PBLK,STB,FAN); */

module x347(PXD, // [9:0] - host interface, will be redefined for 14-bit data
            DCLK,
            BPF,
            HACT,
            VACT,
            MRST,
            ARO,
            ARST,
            SCL0,
            SDA0,
            CNVSYNC,
            CNVCLK,
            RUN,       // Low when the FPGA is programmed and running

            CLK0,      // 125MHz clock


            ADCCLK,    // ADC clock
            SHPA,      // other ADC control signals (outputs)
            SHDA,
            SHPB,
            SHDB,
            CLPOB,
            CLPDM,
            PBLK,
            STB,

            ADB,       // ADC channel B data  (14 bits)
            ADA,       // ADC channel A data  (14 bits)
            
            DV,        // DAC data (10 bits)

            DV1,       // DAC channel 1 data strobe
            DV2,       // DAC channel 2 data strobe
            DVLD,      // DAC channels (both) output update to data

            FDTTL,     // CCD Fast Drain (TTL)
            SHTR,      // CCD Electronic Shutter

            CSRG,      // CS for CCD RG DAC
            CSH2,      // CS for CCD Hor Ph 2 DAC
            CSH1,      // CS for CCD Hor Ph 1 DAC
            CSHL,      // CS for CCD Hor Ph Last DAC
            VHSCL,     // SCL  (serial clock) for horizontal phases amplitudes DACs
            VHSDIN,    // SDIN (serial data in) for horizontal phases amplitudes DACs

            RGTTL,     // CCD Reset Gate (TTL)
            H2TTL,     // CCD Horisontal Phase 2 (TTL)
            HLTTL,     // CCD Horisontal Phase Last (TTL)
            H1TTL,     // CCD Horisontal Phase 1 (TTL)

            DAC8D,     // 8-channel DAC (DC biases, horizontal phases biases) serial data in
            DAC8C,     // 8-channel DAC serial clock     
            DAC8FS,    // 8-channel DAC frame strobe
            DAC8LD,    // 8-channel DAC outputs update
            DAC8R,     // 8-channel DAC reset

            ENAPWR,    // enable analog power suppies
            FAN,       // Fan control (on/off)

            LENSPWR,   // enable lens power
            LENSCLKO,  // lens clock output
            LENSDO,    // lens data output
            LENSCLKI,  // lens clock input
            LENSDI,    // lens data input
            EXT_SYNC,   //Extrenal sync I/O
            GP);        // 8-bit GPIO (shutter control) 

    parameter REVNUM=12'h01d; // changed control of power to parts of the camera
//    parameter REVNUM=12'h01c; // alternating (odd/even) shutter sequence
//    parameter REVNUM=12'h01b; // ext sync i/o
//    parameter REVNUM=12'h01a; // clock phase adjustment
//    parameter REVNUM=12'h019; // mech. shutter uses pixel clock/4
//    parameter REVNUM=12'h017; // adding shutter sequencer
//    parameter REVNUM=12'h016; // mirrored
//    parameter REVNUM=12'h015; // changed merge_streams to use large BlockRAM of Spartan 3E
//    parameter REVNUM=12'h014; // fixed fan polarity
//    parameter REVNUM=12'h011; // porting to 347 from 324
//    parameter REVNUM=8'h02; // added rtc
//    parameter REVNUM=8'h01; // will be read out through bits 11:4 of loc 0 (1 lsb - output number)

  output  [ 9:0] PXD; // [9:0] - host interface - use as DA[13:4]
  output        DCLK;      // FSA
  output        BPF;       // DCLKA
  output        HACT;      // DA[0]
  output        VACT;      // DA[1]
  input         MRST;      // not yet used ***
  input         ARO;       // SA0
  input         ARST;      // SB0
  inout         SCL0;      // 
  inout         SDA0;      // SI0
  output        CNVSYNC;   // DA[2]
  output        CNVCLK;    // DA[3]
  output        RUN;       // Low when the FPGA is programmed and running

  input         CLK0;      // 125MHz clock


  output        ADCCLK;    // ADC clock
  output        SHPA;      // other ADC control signals (outputs)
  output        SHDA;
  output        SHPB;
  output        SHDB;
  output        CLPOB;
  output        CLPDM;
  output        PBLK;
  output        STB;

  input  [13:0] ADB;       // ADC channel B data  (14 bits)
  input  [13:0] ADA;       // ADC channel A data  (14 bits)
            
  output [ 9:0] DV;        // DAC data (10 bits)

  output        DV1;       // DAC channel 1 data strobe
  output        DV2;       // DAC channel 2 data strobe
  output        DVLD;      // DAC channels (both) output update to data

  output        FDTTL;     // CCD Fast Drain (TTL)
  output        SHTR;      // CCD Electronic Shutter

  output        CSRG;      // CS for CCD RG DAC
  output        CSH2;      // CS for CCD Hor Ph 2 DAC
  output        CSH1;      // CS for CCD Hor Ph 1 DAC
  output        CSHL;      // CS for CCD Hor Ph Last DAC
  output        VHSCL;     // SCL  (serial clock) for horizontal phases amplitudes DACs
  output        VHSDIN;    // SDIN (serial data in) for horizontal phases amplitudes DACs

  output        RGTTL;     // CCD Reset Gate (TTL)
  output        H2TTL;     // CCD Horisontal Phase 2 (TTL)
  output        HLTTL;     // CCD Horisontal Phase Last (TTL)
  output        H1TTL;     // CCD Horisontal Phase 1 (TTL)

  output        DAC8D;     // 8-channel DAC (DC biases, horizontal phases biases) serial data in
  output        DAC8C;     // 8-channel DAC serial clock     
  output        DAC8FS;    // 8-channel DAC frame strobe
  output        DAC8LD;    // 8-channel DAC outputs update
  output        DAC8R;     // 8-channel DAC reset

  output        ENAPWR;    // enable analog power suppies
  output        FAN;       // Fan control (on/off)

  output        LENSPWR;   // enable lens power
  output        LENSCLKO;  // lens clock output
  output        LENSDO;    // lens data output
  input         LENSCLKI;  // lens clock input
  input         LENSDI;    // lens data input

  inout  [ 7:0] GP;        // 8-bit GPIO (shutter control) 
  inout         EXT_SYNC;

// i/os in x324
  wire           CLK0;
  wire    [13:0] DA;
//ienapwr 
 assign PXD[9:0] = DA[13:4];
  reg [9:0]      test_cntr;
  


  assign CNVCLK   = DA[3];
  assign CNVSYNC  = DA[2];
  assign VACT     = DA[1];
  assign HACT     = DA[0];

  wire           FSA;
  wire           DCLKA;

  assign DCLK     = FSA;
  assign BPF      = DCLKA;

  wire [2:0] run_d;
 // assign RUN     = 1'b0;       // Low when the FPGA is programmed and running

  wire  [9:0] DV;
  wire        DV1;
  wire        DV2;
  wire        DVLD;
  wire        DAC8C;
  wire        DAC8D;
  wire        DAC8FS;
  wire        DAC8LD;
  wire        DAC8R;
  wire        SHTR;
  wire        FDTTL;
  wire        CSRG;
  wire        CSH1;
  wire        CSHL;
  wire        CSH2;
  wire        VHSCL;
  wire        VHSDIN;
  wire        H1TTL;
  wire        HLTTL;
  wire        H2TTL;
  wire        RGTTL;
  wire        ENAPWR;
  wire  [7:0] GP;
  wire        EXT_SYNC;
  wire        ADCCLK;
  wire        SHPA;
  wire        SHDA;
  wire        SHPB;
  wire        SHDB;
  wire        CLPOB;
  wire        CLPDM;
  wire        PBLK;
  wire        STB;

// internal wires
  wire        c125;

  wire  shtrmode; // 1 - use mechanical shutter (first pair of toggles)
  wire  shtralt;  // 0 - use single sequence, 1  alternate odd/even

  wire  mshutter_start;  // mechanical shutter pulse output
  wire [7:0] mshutter;  
  wire  [3:0] clk_ph;
  wire        clk_div5, pre_cpix;
  wire        cpix;

  reg         clk_sync;
  wire	     dualpr20_nreset;

//  wire        stb_dr;
//  wire        stb_dw;
  reg   [9:0] addr_m;
  reg  [15:0] dwr_m;
  wire [15:0] drd_m; // only 12 LSB read out
  wire [15:0] drd_mc; // only 12 LSB read out, RAM combined with REVNUM/channel number
reg       stb_dr_m,  stb_dw_m,  stb_wa_m;
wire      stb_dr_m0, stb_dw_m0, stb_wa_m0;
reg       stb_dr_d; // delayed stb_dr_m
wire      stb_dr_d0; // resyncing
  wire ram_en_m;
  reg [15:0] dwr;
  reg  [9:0] addr;

  wire cs_monitor_m;
  wire cs_dac8d;
  wire cs_dac8ld;
  wire cs_max5438;
  wire cs_pwr;
  wire cs_dac2900;
  wire cs_seq;
  wire cs_shtr;
  wire cs_expose;
  wire cs_gains;
  wire cs_windows;
  wire cs_rtc;
// monitor some signal (now - just "slave is waiting") through the serial output (until first transition on SDAx)


  wire cs_ll; // half line length, lsb - adjust pixel shift by 1
  wire cs_timing;
  wire cs_stay_alive; // write timings for keepeng analog power on
  wire cs_fan;        // write fan control: 0 - always off, 1 - with analog power, 2,3 - always on

  wire         ienseq;
  wire         en_h;
  wire  [2:0]  seq_v_ph;
  wire         seq_v_stb;
  wire         seq_fd;
  wire         seq_shtr;
  wire         seq_frame;
  wire         seq_line;
  wire         seq_expos;

// synchronized to 125 MHz
  reg          en_h_s;
  reg          seq_line_s;
  reg          seq_v_stb_s;
  reg [2:0]    seq_v_ph_s;
  reg          seq_fd_s;
  reg          seq_shtr_s;
  reg          seq_frame_s;

  wire ienapwr;
//  wire pre_enapwr;
  wire [1:0] pwr_mode; // bit 0 - old "pre_enapwr" - enable when triggered and stay for timeout befoe turning off
							  // bit 1 - enable shutter charger
  
  wire idac8d, idac8fs, idac8ld,idac8clken;
  wire iwhscl,iwhsdin,icsrg,icsh1,icshl,icsh2;
  wire [9:0] idv;
  wire       iwrt1;
  wire       iwrt2;
  wire       ild;
wire       iclk2x,clk2x,iclkitf;
wire	[3:0]	fan_ctrl;
wire        FAN;

wire dummy; // connect it somewhere;
wire iaro, iarst, imrst;
wire pre_clk_sync;
wire        c125i;

wire ext_out; //1 - output sync, 0 - tristate
wire ext_in; 
wire start_seq;
//synthesis translate_off
 defparam i_stay_alive.N24999=15'h0007;
//synthesis translate_on
//seq_frame;
wire sync_out; // send low level to sync output when 1
assign sync_out=0;
wire ext_sync_en; // enable external trigger
reg ext_sngl;    // external trigger, single cycle
reg [1:0] pre_ext_sngl;
reg [1:0] frame_end;
reg       ext_sync_en_single; // enable external trigger once
reg start_imacq; // start image acquisition
reg mask_ext;   // mask external trigger until frame is done

reg fan_pulse; //5usec output pulse on the fan out - triggered by software
reg [6:0] fan_pulse_cntr;
// delay is needed as CCLK stays INPUT for 4 cycles after DONE goes high (contrary to datsheet)
// without this delay (actually it is one too many)

  FD_1 i_run_d0 (.C(iaro),.D(1'b1),    .Q(run_d[0]));
  FD_1 i_run_d1 (.C(iaro),.D(run_d[0]),.Q(run_d[1]));
  FD_1 i_run_d2 (.C(iaro),.D(run_d[0]),.Q(run_d[2]));
  OBUF i_RUN (.I(~run_d[2]),.O(RUN));


assign dummy= iaro | iarst | imrst;

always @ (posedge c125) clk_sync <= pre_clk_sync;
  IBUF   i_c125i (.I(CLK0),  .O(c125i));
  BUFG   i_c125  (.I(c125i), .O(c125));
//TODO: switch interface to 25MHz?
/*
wire        c25mhz;
wire  [2:0] div_by_5;
  FD i_div_by_5_0 (.C(c125), .D(~div_by_5[2] & ~div_by_5[0]),                .Q(div_by_5[0]));
  FD i_div_by_5_1 (.C(c125), .D(~div_by_5[2] & (div_by_5[1] ^ div_by_5[0])), .Q(div_by_5[1]));
  FD i_div_by_5_2 (.C(c125), .D(div_by_5[1]),                                  .Q(div_by_5[2]));
*/
  FD i_dualpr20_nreset (.C(cpix), .D(1'b1), .Q(dualpr20_nreset));


  FDE   i_fan_ctrl0 (.C(cpix), .CE(cs_fan && stb_dw), .D(dwr[0]), .Q(fan_ctrl[0]));
  FDE   i_fan_ctrl1 (.C(cpix), .CE(cs_fan && stb_dw), .D(dwr[1]), .Q(fan_ctrl[1]));
  FDE   i_fan_ctrl2 (.C(cpix), .CE(cs_fan && stb_dw), .D(dwr[2]), .Q(fan_ctrl[2]));
  FDE   i_fan_ctrl3 (.C(cpix), .CE(cs_fan && stb_dw), .D(dwr[3]), .Q(fan_ctrl[3]));

assign  shtrmode=~fan_ctrl[2]; // 1 - use mechanical shutter (first pair of toggles)
assign  shtralt=  fan_ctrl[3];  // 0 - use single sequence, 1  alternate odd/even
  OBUF  i_fan (.I(fan_pulse || (&fan_ctrl[1:0] ? mshutter_start : ((fan_ctrl[1] || (fan_ctrl[0] && ienapwr) )))), .O(FAN));
  
  IBUF  i_LENSCLKI (.I(LENSCLKI),  .O(ilensclki));
  IBUF  i_LENSDI   (.I(LENSDI),    .O(ilensdi));
  OBUF  i_LENSPWR  (.I(dummy),     .O(LENSPWR));
  OBUF  i_LENSCLKO (.I(ilensclki), .O(LENSCLKO));
  OBUF  i_LENSDO   (.I(ilensdi),    .O(LENSDO));
  
  assign cs_monitor_m = (addr_m[9:0]==10'h01); // serial output will monitor selected channel (now - just one) after writing until first SDAx transition 
  assign cs_dac8d = ((addr[9:4]== 6'h02) && (addr[3:2]!=2'b11)); // write to DAC8 - output registers, control registers 0x20..0x2b
  assign cs_dac8ld = (addr[9:1]== 9'h16); // disable (0x2c) and enable (0x2d) copying DAC8 registers to output(s)
  assign cs_max5438= (addr[9:2]== 8'h0c); // write to max5438 (4 of them) - 0x30..0x33
//  assign cs_pwr =    (addr[9:1]== 9'h8 ); // disable (0x10) and enable (0x11) power supplies. Now actual on/off will be controlled by timer
  assign cs_pwr =    (addr[9:0]== 10'h10 ); // 0x10 - power modes:
                                            // bits [1:0] 2 - reset, 3 - set pre_enapwr- enable when triggered and stay for timeout befoe turning off
                                            // bits [3:2] 2 - reset, 3 - set "enable power immediately"
							                       // bits [5:4] 2 - reset, 3 - set "enable shutter charger"
  
  assign cs_dac2900= (addr[9:4]== 6'h4 ); // DAC2900 0x40..0x4f

  assign cs_seq =    (addr[9:8]== 2'h1 ); // 0x100..0x1ff - write sequencer program memory
  assign cs_shtr =   (addr[9:8]== 2'h2 ); // 0x200..0x2ff - write sequencer program memory
  assign cs_expose=  (addr[9:0]==10'h14); // 0x14 - start exposure (new: d=0 - start, d=2 - disabel ext, d=3 - en ext)
  assign cs_gains=   (addr[9:4]== 6'h5); // 0x50..58 - zero subtract, 58-5f - gains(16 bits, 0x1000 - unity)
  assign cs_windows= (addr[9:3]== 7'hc); // 0x60..63 - window 1, 0x64..0x67 - window 2
  assign cs_ll=      (addr[9:0]==10'h68); // 0x68 write half line length
//  assign cs_timing=(addr[9:4]==6'h7); // 0x70..0x7f
  assign cs_timing=(addr[9:4]==6'h7) || (addr[9:1]==9'h37); // 0x6e..0x7f
//  assign cs_stay_alive=(addr[9:2]==8'h20); // 0x80.. keep power (in 0.1 sec), 81 - sequencer on delay (in 0.001 sec), 82 - power off delay (in 0.001 sec)
  assign cs_stay_alive=(addr[9:3]==7'h10); // 0x80..0x87 keep power (in 0.1 sec), 81 - sequencer on delay (in 0.001 sec), 82 - power off delay (in 0.001 sec)
                                           // 84 - PWM period (in pixel clocks), 85 - PWM start (= period - longest)
                                           // 86 - PWM decrement (each period)
  assign cs_fan=      (addr[9:0]==10'h16);        // write fan control: 0 - always off, 1 - with analog power, 2,3 - always on
  assign cs_rtc=      (addr[9:3]==7'h14); // 0x0a0..0x0a7
/*
      0          1          2          3          4          5          6          7          8          9          a          b          c          d          e          f
00          monitor_m   slavewaiting   -          -          -          -          -          -          -          -          -          -          -          -          - 
            monitor_s
10 pwr_mode      -      (dis_seq)  (en_seq)  start_exp      -       fan_ctrl      -          -          -          -          -          -          -          -          - 
20 dac8d_0    dac8d_1    dac8d_2    dac8d_3    dac8d_4    dac8d_5    dac8d_6    dac8d_7    dac8d_8    dac8d_9    dac8d_a    dac8d_b  dac8ld_off  dac8ld_on      -          - 
30 max5438_0  max5438_1  max5438_2  max5438_3     -          -          -          -          -          -          -          -          -          -          -          - 
40 dac2900_0  dac2900_1  dac2900_2  dac2900_3  dac2900_4  dac2900_5  dac2900_6  dac2900_7  dac2900_8  dac2900_9 dac2900_10 dac2900_11 dac2900_12 dac2900_13 dac2900_14 dac2900_15
50 sub_a0     sub_a1     sub_a2     sub_a3     sub_b0     sub_b1     sub_b2     sub_b3     gain_a0    gain_a1   gain_a2    gain_a3    gain_b0    gain_b1    gain_b2    gain_b3
60 win_m_0    win_m_1    win_m_2    win_m_3    win_s_0    win_s_1    win_s_2    win_s_3    half_line     -          -          -          -          -      timing16   timing17 
70 timing0    timing1    timing2    timing3    timing4    timing5    timing6    timing7    timing8    timing9    timing10   timing11  timing12   timing13   timing14   timing15
80 keep_pwr   seq_on_dly pwr_off_dly   -          -          -          -          -          -          -          -          -          -          -          -          - 
90
a0
b0
c0
d0
e0
f0
100..1ff - CCD cequencer
200..2ff - shutter sequencer
200 - 27f - first, third, (odd) exposures (or all)
280 - 2ff - second, fourth (even) exposure
*/


// modified with dualpr20 to give an extra cycle to blockram, so the data is expected valid next cycle aftre rd (when a is already different)
  reg         read_0m, read_time_m;
  reg  [ 7:0] tim_mux_m;  // multiplexed timer data (selected byte), master
  wire [31:0] systime_sec;
  wire [19:0] systime_usec;
  wire [31:0] frametime_sec;
  wire [19:0] frametime_usec;
//  wire   
//  always @ (posedge cpix) begin
  always @ (posedge cpix) if (stb_dr_m) begin
    read_0m <= (addr_m[9:0]==8'h0);
    read_time_m <= (addr_m[9:4]==6'h0b); // 0b0..0bf - read timer
    case (addr_m[3:0])
     4'h0:  tim_mux_m[7:0] <= systime_sec [31:24];
     4'h1:  tim_mux_m[7:0] <= systime_sec [23:16];
     4'h2:  tim_mux_m[7:0] <= systime_sec [15: 8];
     4'h3:  tim_mux_m[7:0] <= systime_sec [ 7: 0];
     4'h4:  tim_mux_m[7:0] <= 8'h0;//systime_usec[31:24];
     4'h5:  tim_mux_m[7:0] <= {4'h0,systime_usec[19:16]}; //systime_usec[23:16];
     4'h6:  tim_mux_m[7:0] <= systime_usec[15: 8];
     4'h7:  tim_mux_m[7:0] <= systime_usec[ 7: 0];
     4'h8:  tim_mux_m[7:0] <= frametime_sec [31:24];
     4'h9:  tim_mux_m[7:0] <= frametime_sec [23:16];
     4'ha:  tim_mux_m[7:0] <= frametime_sec [15: 8];
     4'hb:  tim_mux_m[7:0] <= frametime_sec [ 7: 0];
     4'hc:  tim_mux_m[7:0] <= 8'h0;//frametime_usec[31:24];
     4'hd:  tim_mux_m[7:0] <= {4'h0,frametime_usec[19:16]}; //frametime_usec[23:16];
     4'he:  tim_mux_m[7:0] <= frametime_usec[15: 8];
     4'hf:  tim_mux_m[7:0] <= frametime_usec[ 7: 0];
    endcase
  end
  assign drd_mc[15:0]=read_time_m?{8'b0,tim_mux_m[7:0]}:(read_0m?{REVNUM,ienapwr,3'h0}:{drd_m[15:0]});

// now - monitor for just one signal, may be in increased

 rtc324 i_rtc324 (.clk125(c125),                // 125MHz clock, posedge
                  .sclk(cpix),                  // system clock (pixel clock), posedge
                  .exposure(seq_expos),         // sensor exposure - start will be used to latch frame time stamp
                  .we(cs_rtc && stb_dw),        // sync to sclk                
                  .wa(addr[2:0]),               // [2:0] write address
                                                // 0 - write seconds high word - will write all other (actual write will happen after writing low word microseconds)
                                                // 1 - write seconds low word (actual write will happen after writing low word microseconds)
                                                // 2 - write microseconds high word (actual write will happen after writing low word microseconds)
                                                // 3 - write microseconds low word and set the clock seconds, microseconds
                                                // 4 - write correction (16 bit, signed
                                                // 5 - nop, just latch the output 32+20 bits to read out).
                  .wd(dwr[15:0]),               // [15:0] data to write, valid 1 cycle after pre_we, wa
                  .ssec(systime_sec[31:0]),     // [31:0] latched system time, seconds
                  .susec(systime_usec[19:0]),   // [19:0] latched system time, usec -  0..999999 (0..0xf423f)
                  .fsec(frametime_sec[31:0]),   // [19:0] latched exposure start, seconds
                  .fusec(frametime_usec[19:0]));// [31:0] latched exposure start, microseconds
 
// memory to test dualpr20
//assign ram_en_m=(stb_dr_m || stb_dw_m) && !addr_m[9] && !addr_m[8];
assign ram_en_m=(stb_dr_m || stb_dw_m);
  wire        i2c_scl;             // SCL from master
  wire        i2c_sdami;           // SDA from master input
  wire        i2c_sdamo;           // SDA to master output
  wire        i2c_sdamen;          // SDA to master output enable
  wire [15:0] i2c_sr;              // 16-bit shift register output (skipped ACKN)
  wire        i2c_wra_stb;         // single-cycle strobe when sr contains register+slave address to read/write
  wire        i2c_wra_ackn;        // acknowledge address (active high input)
  wire        i2c_wrd_stb;         // single-cycle strobe when sr[7:0] contains byte to write (st[16:9] may have previous/MS byte)
  wire        i2c_wrd_ackn;        // acknowledge for the byte/word written
  wire        i2c_rd_req;          // request read byte (ackn from master), strobe after SCL goes high
  wire  [7:0] i2c_rdat;            // 8-bit data to send to master
  reg         i2c_rd_stb;          // rdat is updated
  wire        i2c_start;           // start strobe (just in case)
  wire        i2c_stop;            // stop strobe (some commands will be triggered now (i.e. connecting/disconnecting bridge to external slave)

  reg         i2c_wr_lo;          // 0 - high byte, 1 - low byte
  reg         i2c_rd_hi;          // 0 - low byte, 1 - high byte
//  reg  [15:0] i2c_rd16;           // 16-bit data to send over i2c (high byte first)
  assign i2c_wra_ackn=1'b1;
  assign i2c_wrd_ackn=1'b1;
//  assign      i2c_rdat[7:0]= i2c_rd_hi ? i2c_rd16[15:8] : i2c_rd16[7:0];
  assign      i2c_rdat[7:0]= i2c_rd_hi ? drd_mc[15:8] : drd_mc[7:0];
// sr[15:0] will not change SCL=0 time before wr?_stb and will not change for a full bit interval after
// so it is safe to use sr[] after the wr?_stb
//drd_mc[15:0]
  RAMB16_S18_S18  RAMB16_S18_S18_inst (
      .DOA(drd_m[15:0]),      // Port A 16-bit Data Output
      .DOPA(),                // Port A 2-bit Parity Output
      .ADDRA(addr_m[9:0]),   // Port A 10-bit Address Input
      .CLKA(cpix),            // Port A Clock
      .DIA(dwr_m[15:0]),      // Port A 16-bit Data Input
      .DIPA(2'h0),            // Port A 2-bit parity Input
      .ENA(ram_en_m),         // Port A RAM Enable Input
      .SSRA(1'b0),            // Port A Synchronous Set/Reset Input
      .WEA(stb_dw_m),         // Port A Write Enable Input
      
      .DOB(),      // Port B 16-bit Data Output
      .DOPB(),    // Port B 2-bit Parity Output
      .ADDRB(10'h0),  // Port B 10-bit Address Input
      .CLKB(1'b0),    // Port B Clock
      .DIB(16'h0),      // Port B 16-bit Data Input
      .DIPB(2'h0),    // Port-B 2-bit parity Input
      .ENB(1'b0),      // Port B RAM Enable Input
      .SSRB(1'b0),    // Port B Synchronous Set/Reset Input
      .WEB(1'b0)       // Port B Write Enable Input

   );

  i2csbr i_i2csbr (.clk(c125),                  // global clock (30-150MHz?)
                   .slave_en(1'b0),                 // enable slave bus (may be disabled while fast/non-standard communications with master)
                   .scl(i2c_scl),               // SCL from master
                   .scl_en(),                   // enable sclk output to external slave device
                   .sdami(i2c_sdami),           // SDA from master input
                   .sdamo(i2c_sdamo),           // SDA to master output
                   .sdamen(i2c_sdamen),         // SDA to master output enable
                   .sdasi(1'b0),                // SDA from external slave input
                   .sdaso(),                    // SDA to external slave output
                   .sdasen(),                   // SDA to external slave output enable
                   .sr(i2c_sr[15:0]),           // 16-bit shift register output (skipped ACKN)
                   .slava_ackn(i2c_sr[7:3]==5'h04), // will ackn. (using 7 MSBs from sr)
                   .wra_stb(i2c_wra_stb),       // single-cycle strobe when sr contains register+slave address to read/write
                   .wra_ackn(i2c_wra_ackn),     // acknowledge address (active high input)
                   .wrd_stb(i2c_wrd_stb),       // single-cycle strobe when sr[7:0] contains byte to write (st[16:9] may have previous/MS byte)
                   .wrd_ackn(i2c_wrd_ackn),     // acknowledge for the byte/word written
                   .rd_req(i2c_rd_req),         // request read byte (ackn from master), strobe after SCL goes high
                   .rdat(i2c_rdat[7:0]),        // 8-bit data to send to master
                   .rd_stb(i2c_rd_stb),         // rdat is updated
                   .start(i2c_start),           // start strobe (just in case)
                   .stop(i2c_stop));            // stop strobe (some commands will be triggered now (i.e. connecting/disconnecting bridge to external slave)

// synchronize 125MHz i2csbr I/Os with slower (cpix) internal signals
// reg       stb_dr_m, stb_dw_m;
//  reg   [9:0] addr_m;
//  reg  [15:0] dwr_m;
  always @ (posedge c125) begin
//    if      (i2c_wra_stb) addr_m[9:0] <= {sr[10:9],sr[7:0]};
    if      (i2c_wra_stb) i2c_wr_lo  <= 1'b0;
    else if (i2c_wrd_stb) i2c_wr_lo  <= ~i2c_wr_lo;
    
    if      (i2c_wrd_stb & i2c_wr_lo) dwr_m[15:0] <= i2c_sr[15:0];
    if      (i2c_wra_stb) i2c_rd_hi  <= 1'b0;
    else if (i2c_rd_req)  i2c_rd_hi  <= ~i2c_rd_hi;
    
    i2c_rd_stb <= stb_dr_d0 | (i2c_rd_hi & i2c_rd_req);
  end
  FDCE i_stb_wa_m0 (.Q(stb_wa_m0), .C(c125), .CE(i2c_wra_stb), .CLR(stb_wa_m), .D(1'b1));
  FDCE i_stb_dw_m0 (.Q(stb_dw_m0), .C(c125), .CE(i2c_wrd_stb & i2c_wr_lo), .CLR(stb_dw_m), .D(1'b1));
  FDCE i_stb_dr_m0 (.Q(stb_dr_m0), .C(c125), .CE(i2c_rd_req & ~i2c_rd_hi),  .CLR(stb_dr_m), .D(1'b1));

  always @ (posedge cpix) begin
    stb_wa_m <= stb_wa_m0;
    stb_dw_m <= stb_dw_m0;
    stb_dr_m <= stb_dr_m0;
    stb_dr_d <= stb_dr_m;
   if      (stb_wa_m)             addr_m[9:0] <= {i2c_sr[10:9],i2c_sr[7:0]};
   else if (stb_dw_m | stb_dr_m)  addr_m[9:0] <=   addr_m[9:0] +1 ;
  end
  FDCE i_stb_dr_d0 (.Q(stb_dr_d0), .C(cpix), .CE(stb_dr_d),  .CLR(i2c_rd_stb), .D(1'b1));

  FD i_stb_dw(.C(cpix), .D(stb_dw_m), .Q(stb_dw));

   always @ (posedge cpix) 
     if (stb_dw_m) begin
       addr[9:0] <= addr_m[9:0];
       dwr[15:0] <= dwr_m[15:0];
     end

//  reg   [9:0] addr_m;
//  reg  [15:0] dwr_m;




  PULLUP i_sdam_pu (.O(SDA0));
  PULLUP i_sclm_pu (.O(SCL0));
  IOBUF    i_sclm     (.I(1'b0),    .T(1'b1),        .O(i2c_scl), .IO(SCL0));
  IOBUF    i_sdam     (.I(i2c_sdamo), .T(~i2c_sdamen), .O(i2c_sdami), .IO(SDA0));





//wire [1:0] pwr_mode;
 // bit 0 - old "pre_enapwr" - enable when triggered and stay for timeout befoe turning off
							// bit 1 - enable shutter charger
  FDE  i_pwr_mode0 (.C(cpix), .CE(cs_pwr && dwr[1] && stb_dw), .D (dwr[0]), .Q(pwr_mode[0]));
  FDE  i_pwr_mode1 (.C(cpix), .CE(cs_pwr && dwr[3] && stb_dw), .D (dwr[2]), .Q(pwr_mode[1]));
							
  OBUF  i_ENAPWR     (.I(ienapwr ),  .O(ENAPWR));



tlv5630 i_tlv5630(.clk(cpix),	  //25MHz, output SCLK generated externally to match ThD=5ns
                  .we(stb_dw),     // WE from serial interface
					   .a(addr[3:0]),   // [3:0] - register select , LSB also LD on/off select
					   .d(dwr[11:0]),   //[11:0] - data 
					   .cswr(cs_dac8d),// writing to register (0x20..0x29)
					   .csld(cs_dac8ld),// writing to LD on/off(0x10-off, 0x11 - on)
					   .sd(idac8d),  // serial data to DAC
					   .fs(idac8fs),  // frame sync
					   .ld(idac8ld), // LD enable (active low)
						.clken(dac8clken));
  OBUF  i_DAC8D  (.I(idac8d),  .O(DAC8D));
  OBUF  i_DAC8FS (.I(idac8fs), .O(DAC8FS));
  OBUF  i_DAC8LD (.I(idac8ld), .O(DAC8LD));
  OBUF  i_DAC8C  (.I(!pre_cpix && dac8clken),    .O(DAC8C));
  OBUF  i_DAC8R  (.I(1'b1),    .O(DAC8R));
max5438 i_max5438 (.clk(cpix), // 25Mz
                   .we(stb_dw),  // 
					    .a(addr[1:0]),   // [1:0] DAC select
					    .d(dwr[7:0]),   // [7:0] data (MSB - not used
					    .cswr(cs_max5438),// decoded address - enable DAC write
					    .scl(iwhscl), // serial clock to DACs
					    .sda(iwhsdin), // serial data to DACs
					    .cs({icsrg,icsh1,icshl,icsh2})); // [3:0] - enable individual DAC, active low

  OBUF  i_VHSCL  (.I(iwhscl),  .O(VHSCL));
  OBUF  i_VHSDIN (.I(iwhsdin), .O(VHSDIN));
  OBUF  i_CSRG   (.I(icsrg),   .O(CSRG));
  OBUF  i_CSH1   (.I(icsh1),   .O(CSH1));
  OBUF  i_CSHL   (.I(icshl),   .O(CSHL));
  OBUF  i_CSH2   (.I(icsh2),   .O(CSH2));


dac2900 i_dac2900(.oclk(c125),   //125MHz)
                  .a(seq_v_ph_s[2:0]),      // [2:0] - output select
					   .stb(seq_v_stb_s && clk_sync),		// synch to clk
					   .do(idv[9:0]),		// [9:0]
					   .wrt1(iwrt1),   //
					   .wrt2(iwrt2),   //
					   .ld(ild),     //
// programming interface
					   .clk(cpix),    // write clock, 25MHz
					   .we(stb_dw),
                  .addr(addr[3:0]),    // [3:0] write address, 2*n -> V1, 2*n+1-> V2
					   .d(dwr[9:0]),
					   .cswr(cs_dac2900));
sdo1  i_DV0(.c(c125),.d(idv[0]),.q(DV[0]));
sdo1  i_DV1(.c(c125),.d(idv[1]),.q(DV[1]));
sdo1  i_DV2(.c(c125),.d(idv[2]),.q(DV[2]));
sdo1  i_DV3(.c(c125),.d(idv[3]),.q(DV[3]));
sdo1  i_DV4(.c(c125),.d(idv[4]),.q(DV[4]));
sdo1  i_DV5(.c(c125),.d(idv[5]),.q(DV[5]));
sdo1  i_DV6(.c(c125),.d(idv[6]),.q(DV[6]));
sdo1  i_DV7(.c(c125),.d(idv[7]),.q(DV[7]));
sdo1  i_DV8(.c(c125),.d(idv[8]),.q(DV[8]));
sdo1  i_DV9(.c(c125),.d(idv[9]),.q(DV[9]));

sdo1  i_DVW1(.c(c125),.d(iwrt1),.q(DV1));
sdo1  i_DVW2(.c(c125),.d(iwrt2),.q(DV2));
sdo1  i_DVLD(.c(c125),.d(ild),  .q(DVLD));

/*
wire ext_out; //1 - output sync, 0 - tristate
wire ext_in; 
wire start_seq;
//synthesis translate_off
 defparam i_stay_alive.N24999=15'h0007;
//synthesis translate_on
//seq_frame;
wire ext_sync_en; // enable external trigger
reg ext_sngl;    // external trigger, single cycle
reg [1:0] pre_ext_sngl;
reg [1:0] frame_end;
reg       ext_sync_en_single; // enable external trigger once
reg start_imacq; // start image acquisition
reg mask_ext;   // mask external trigger until frame is done
reg fan_pulse; //5usec output pulse on the fan out - triggered by software
reg [6:0] fan_pulse_cntr;

*/
wire start_fan_pulse=cs_expose && stb_dw && dwr[2];
  FDE i_ext_sync_en (.Q(ext_sync_en), .C(cpix), .CE(cs_expose && stb_dw && dwr[1]), .D(dwr[0]));
  always @ (posedge cpix) begin
    start_imacq <=(cs_expose && stb_dw && (dwr[2:0]==3'b0) ) ||
                  (((ext_sync_en && !mask_ext) || ext_sync_en_single) && ext_sngl);
    pre_ext_sngl <= {pre_ext_sngl[0],ext_in};
	 ext_sngl <= (pre_ext_sngl[1:0]==2'h1);
	 frame_end <={(!seq_frame && frame_end[0]) , seq_frame}; //frame_end[1] - single cycle after frame end
	 ext_sync_en_single <= (cs_expose && stb_dw && (dwr[2:0]==3'h1)) || (ext_sync_en_single && !start_imacq);
// mask activated by external trigger,	 disabled by end of acquisition and any write to cs_expose
	 mask_ext <= start_imacq || (mask_ext && !(cs_expose && stb_dw) && !frame_end[1]);
	 fan_pulse <= start_fan_pulse || (fan_pulse && (fan_pulse_cntr[6:0]!=7'h0));
	 if (!fan_pulse || start_fan_pulse) fan_pulse_cntr[6:0] <= 7'h7c;
	 else fan_pulse_cntr[6:0] <= fan_pulse_cntr[6:0]-1;
  end

stay_alive i_stay_alive(.clk(cpix),                //25MHz
                        .we(cs_stay_alive&& stb_dw),    
                        .a(addr[2:0]),              // [1:0] reg address (00 - watchdog), 01 - seq. on delay, 10 - power off delay (0.001 sec)
                        .d(dwr[15:0]),              // [15:0]  (watchdog - 16 bits (65.536 sec max), others - 10 bits (1.024 sec max)
                        .en(pwr_mode[0]),            // enable (was power on)
                        .trig(start_imacq), // get picture cmd (1 cycle)
                        .pwren(ienapwr),            // power enable output 
                        .seqen(ienseq),             // enable sequencer
                        .seqtrig(start_seq));       // trigger sequencer

ccd_seq i_ccd_seq(.clk(cpix),	// pixel clock
                .wr(cs_seq && stb_dw),  // write enable, sync to clk
					 .wa(addr[7:0]),  // wa
					 .wd(dwr[15:0]),  // 16 bit data (now - only exposure time)
					 .en(ienseq), //  enable sequencer
					 .strt(start_seq), // start exposure, when finished - readout
					 .en_h(en_h),	// enable horizontal phases
					 .v_ph(seq_v_ph[2:0]), // [2:0] - number of prewritten vertical phases combination
					 .v_stb(seq_v_stb),   // change vertical phases to v_phase,
					 .fd(seq_fd),		 // activate "fast drain"
					 .shtr(seq_shtr),    // activate shutter
					 .frame(seq_frame), // frame active
					 .line(seq_line),    // = en_h && frame. all latency will be added externally
					 .expos(seq_expos),   // exposure in progress.
                .shtrmode(shtrmode), // 1 - use mechanical shutter (first pair of toggles)
                .mshutter(mshutter_start)  // mechanical shutter pulse output
                 );
                 
shutter_seq i_shutter_seq (.clk(cpix), // "pixel" clock (25MHz)
                           .wr(cs_shtr && stb_dw),  // write enable, sync to clk
                           .wa(addr[7:0]),  // wa
                           .wd(dwr[15:0]),  // 16 bit data 
                           .en(ienseq),     // enable sequencer (0 immediately resets output to 8'h0)
									.alt(shtralt),   // alternate odd/even exposures
                           .strt(mshutter_start), // start sequencer (OK to have it > 1 cycle, will use leading edge)
                           .run(),  // sequencer is running 
                           .q(mshutter[7:0]));   // 8-bit output (idle - 8'h0)                 
                 

reg en_h_s1; // 40 ns long
always @ (posedge c125)
  if (clk_sync) begin
    seq_line_s  <= seq_line;
    en_h_s      <= en_h;
    en_h_s1     <= en_h && !en_h_s;
	 seq_v_stb_s <= seq_v_stb;
	 seq_v_ph_s[2:0] <= seq_v_ph[2:0];
	 seq_fd_s    <= seq_fd;
    seq_shtr_s  <= seq_shtr;
    seq_frame_s <= seq_frame;
  end

wire end_clpdm;
wire start_clpob;
wire mid_clpob;
wire end_clpob;
reg	clpdm1; // copy of CLPDM
reg   clpob1;

wire iclk_adc, clk_adc; // global clock sync to ADC
wire [13:0] iada;
wire [13:0] iadb;
wire [13:0] iaqa;
wire        vact_a,hact_a;
reg         vact_a_d;


always @ (posedge c125) clpdm1 <= (en_h && !en_h_s && clk_sync) || (clpdm1 && !(clk_sync && end_clpdm));

	SRL16E i_end_clpdm   (.Q(end_clpdm),  .A0(1'b0), .A1(1'b1), .A2(1'b0), .A3(1'b0), .CLK(c125),.CE(clk_sync), .D(en_h_s1));	// 3
	SRL16E i_start_clpob (.Q(start_clpob),.A0(1'b1), .A1(1'b1), .A2(1'b0), .A3(1'b0), .CLK(c125),.CE(clk_sync), .D(en_h_s1));	// 4
	SRL16E i_mid_clpob   (.Q(mid_clpob),  .A0(1'b1), .A1(1'b1), .A2(1'b1), .A3(1'b1), .CLK(c125),.CE(clk_sync), .D(en_h_s1));	// 16
	SRL16E i_end_clpob   (.Q(end_clpob),  .A0(1'b1), .A1(1'b0), .A2(1'b1), .A3(1'b0), .CLK(c125),.CE(clk_sync), .D(mid_clpob));	// 6
always @ (posedge c125) if (clk_sync) clpob1 <= en_h && (start_clpob || (!end_clpob && clpob1));





// slow signals
sdo1       i_FD     (.c(c125),.d(!seq_fd_s),  .q(FDTTL));	//EL7156 inverts
sdo1       i_SHTR   (.c(c125),.d(!seq_shtr_s),.q(SHTR)); // active low

// fast signals
out_rg     i_RGTTL  (.c(c125),.d(!( pre_rg && ienseq)),.q(RGTTL)); 	//EL7156 inverts
out_ph1    i_H1TTL  (.c(c125),.d( (pre_ph && en_h_s)), .q(H1TTL));	//EL7156 inverts
out_ph1l   i_HLTTL  (.c(c125),.d( (pre_ph && en_h_s)), .q(HLTTL));	//EL7156 inverts
out_ph2    i_H2TTL  (.c(c125),.d(!(pre_ph && en_h_s)), .q(H2TTL));	//EL7156 inverts
wire iadcclk, ishpa, ishda, ishpb, ishdb;

wire  [4:0] timing_addr=addr[4]?{1'b0,addr[3:0]}:{4'b1000,addr[0]};

timing i_timing(.clk(c125),
                .we(stb_dw && cs_timing),
                .a(timing_addr[4:0]),    // address [3:0]
                                  // 0 -  write  period (default = 4 =>5 cycles, maximum 15->16 cycles). For foolproof everything less than 4 will add 4
                                  // 1 -  write  PH inactive (for period ==5 should be 3)
                                  // 2 - ADCCLK  active at ( 0- 7)( 4-11)( 8-15) ... ( 56.. 63) ns
                                  // 3 - ADCCLK  active at (64-71)(68-75)(72-79) ... (120..127) ns
                                  // 4 - SHPA    active at ( 0- 7)( 4-11)( 8-15) ... ( 56.. 63) ns
                                  // 5 - SHPA    active at (64-71)(68-75)(72-79) ... (120..127) ns
                                  // 6 - SHDA    active at ( 0- 7)( 4-11)( 8-15) ... ( 56.. 63) ns
                                  // 7 - SHDA    active at (64-71)(68-75)(72-79) ... (120..127) ns
                                  // 8 - SHPB    active at ( 0- 7)( 4-11)( 8-15) ... ( 56.. 63) ns
                                  // 9 - SHPB    active at (64-71)(68-75)(72-79) ... (120..127) ns
                                  // a - SHDB    active at ( 0- 7)( 4-11)( 8-15) ... ( 56.. 63) ns
                                  // b - SHDB    active at (64-71)(68-75)(72-79) ... (120..127) ns
                                  // c - clk_adc active at ( 0- 7)( 4-11)( 8-15) ... ( 56.. 63) ns
                                  // d - clk_adc active at (64-71)(68-75)(72-79) ... (120..127) ns
                                  // e - clk_2x  active at ( 0- 7)( 4-11)( 8-15) ... ( 56.. 63) ns
                                  // f - clk_2x  active at (64-71)(68-75)(72-79) ... (120..127) ns
                                  //10 - clk_itf active at ( 0- 7)( 4-11)( 8-15) ... ( 56.. 63) ns
                                  //11 - clk_itf  active at (64-71)(68-75)(72-79) ... (120..127) ns
                .d(dwr[15:0]),    // input data, [15:0]
                .pre_sync(pre_clk_sync),  // 1 cycle long, just before cpix 
					 .cpix(cpix),      // pixel clock (former clk25)
 					 .pre_cpix(pre_cpix), // 1 cycle before cpix
					 .pre_rg(pre_rg),  //1 cycle ahead of RG output, active high
					 .pre_ph(pre_ph), // 1 cycle ahead of PH1 going high, PH2 going low
                .adcclk(iadcclk),
                .shpa(ishpa),
                .shda(ishda),
                .shpb(ishpb),
                .shdb(ishdb),
					 .iclk_adc(iclk_adc), // global clock, normally sync to adcclk
                .iclk2x(iclk2x),  //same as clk2x, but not global
                .iclkitf(iclkitf));  // phase-shifted clk2x - to go to the main board
  reg icpix;
  always @ (posedge c125) icpix <= pre_cpix;
  BUFG   i_cpix (.I(icpix), .O(cpix));
  BUFG   i_clk_adc (.I(iclk_adc), .O(clk_adc));
  BUFG   i_clk2x   (.I(iclk2x),   .O(clk2x)  );

OBUF        i_ADCCLK(.I(iadcclk),.O(ADCCLK));

OBUF        i_SHPA  (.I(ishpa),  .O(SHPA));
OBUF        i_SHDA  (.I(ishda),  .O(SHDA));
OBUF        i_SHPB  (.I(ishpb),  .O(SHPB));
OBUF        i_SHDB  (.I(ishdb),  .O(SHDB));

// add small delay??
sdo1       i_CLPDM  (.c(c125),.d(!((en_h && !en_h_s && clk_sync) || (clpdm1 && !(clk_sync && end_clpdm )))),  .q(CLPDM));
sdo1       i_CLPOB  (.c(c125),.d(!clpob1),  .q(CLPOB));

OBUF       i_PBLK    (.I(seq_frame_s),.O(PBLK));
OBUF       i_STB     (.I(!ienapwr),.O(STB));

//OBUF        i_DCLKA  (.I(iclk2x),  .O(DCLKA));
OBUF        i_DCLKA  (.I(iclkitf),  .O(DCLKA));

always @ (posedge clk2x) vact_a_d <=vact_a;


sdo1       i_FSA    (.c(clk2x),.d(hact_a || (vact_a && !vact_a_d)),  .q(FSA)); // sync to clk2x

pixproc i_pixproc(.wclk(cpix), // interface clock
                  .weg(stb_dw && cs_gains),   // write enable (and cs)
                  .wew(stb_dw && cs_windows),   // write enable (and cs)
                  .we_ll(stb_dw &&  cs_ll),// write half-line-length (LSB - 1 pixel shift). Adjust to merge 2 halves 
					   .a(addr[3:0]),     // [3:0] register address 0..3 - gains channel 1, 4..7 - channel 2, 8..b - left, width,top, height
					   .d(dwr[15:0]),     // [11:0] gains - 8 bits, sizes - 12 bits
					   .aclk(clk_adc),    // ADC/pixel clock
						.clk2x(clk2x),     // output clock, twice ADC, 
					   .frame(seq_frame_s), // high during frame 
					   .line(seq_line_s),     // high during pixel shift 
					   .da(iada[13:0]),   // [13:0] data in, channel A
					   .db(iadb[13:0]),   // [13:0] data in, channel B
					   .qa(iaqa[13:0]),   // [13:0] data out, channel A
//					   .qb(iaqb[13:0]),   // [13:0] data out, channel B
//					   .qb(),   // [13:0] data out, channel B
					   .hact_a(hact_a),   // valid pixel output
//					   .hact_b(hact_b),   // valid pixel output
//					   .hact_b(      ),   // valid pixel output
					   .vact_a(vact_a)   // active frame output
//					   .vact_b(vact_b),   // active frame output
//					   ,.vact_b(      )
                  );  // active frame output

ipadqff i_ada0 (.c(clk_adc),.e(1'b1),.qr(iada[ 0]),.d(ADA[ 0]));
ipadqff i_ada1 (.c(clk_adc),.e(1'b1),.qr(iada[ 1]),.d(ADA[ 1]));
ipadqff i_ada2 (.c(clk_adc),.e(1'b1),.qr(iada[ 2]),.d(ADA[ 2]));
ipadqff i_ada3 (.c(clk_adc),.e(1'b1),.qr(iada[ 3]),.d(ADA[ 3]));
ipadqff i_ada4 (.c(clk_adc),.e(1'b1),.qr(iada[ 4]),.d(ADA[ 4]));
ipadqff i_ada5 (.c(clk_adc),.e(1'b1),.qr(iada[ 5]),.d(ADA[ 5]));
ipadqff i_ada6 (.c(clk_adc),.e(1'b1),.qr(iada[ 6]),.d(ADA[ 6]));
ipadqff i_ada7 (.c(clk_adc),.e(1'b1),.qr(iada[ 7]),.d(ADA[ 7]));
ipadqff i_ada8 (.c(clk_adc),.e(1'b1),.qr(iada[ 8]),.d(ADA[ 8]));
ipadqff i_ada9 (.c(clk_adc),.e(1'b1),.qr(iada[ 9]),.d(ADA[ 9]));
ipadqff i_ada10(.c(clk_adc),.e(1'b1),.qr(iada[10]),.d(ADA[10]));
ipadqff i_ada11(.c(clk_adc),.e(1'b1),.qr(iada[11]),.d(ADA[11]));
ipadqff i_ada12(.c(clk_adc),.e(1'b1),.qr(iada[12]),.d(ADA[12]));
ipadqff i_ada13(.c(clk_adc),.e(1'b1),.qr(iada[13]),.d(ADA[13]));

ipadqff i_adb0 (.c(clk_adc),.e(1'b1),.qr(iadb[ 0]),.d(ADB[ 0]));
ipadqff i_adb1 (.c(clk_adc),.e(1'b1),.qr(iadb[ 1]),.d(ADB[ 1]));
ipadqff i_adb2 (.c(clk_adc),.e(1'b1),.qr(iadb[ 2]),.d(ADB[ 2]));
ipadqff i_adb3 (.c(clk_adc),.e(1'b1),.qr(iadb[ 3]),.d(ADB[ 3]));
ipadqff i_adb4 (.c(clk_adc),.e(1'b1),.qr(iadb[ 4]),.d(ADB[ 4]));
ipadqff i_adb5 (.c(clk_adc),.e(1'b1),.qr(iadb[ 5]),.d(ADB[ 5]));
ipadqff i_adb6 (.c(clk_adc),.e(1'b1),.qr(iadb[ 6]),.d(ADB[ 6]));
ipadqff i_adb7 (.c(clk_adc),.e(1'b1),.qr(iadb[ 7]),.d(ADB[ 7]));
ipadqff i_adb8 (.c(clk_adc),.e(1'b1),.qr(iadb[ 8]),.d(ADB[ 8]));
ipadqff i_adb9 (.c(clk_adc),.e(1'b1),.qr(iadb[ 9]),.d(ADB[ 9]));
ipadqff i_adb10(.c(clk_adc),.e(1'b1),.qr(iadb[10]),.d(ADB[10]));
ipadqff i_adb11(.c(clk_adc),.e(1'b1),.qr(iadb[11]),.d(ADB[11]));
ipadqff i_adb12(.c(clk_adc),.e(1'b1),.qr(iadb[12]),.d(ADB[12]));
ipadqff i_adb13(.c(clk_adc),.e(1'b1),.qr(iadb[13]),.d(ADB[13]));
  always @ (posedge c125) test_cntr[4:0] <= test_cntr[4:0]+1;
  always @ (posedge cpix) test_cntr[9:5] <= test_cntr[9:5]+1;
//wire [13:0] iaqaX = ienapwr?iaqa[13:0]:{test_cntr[9:0],4'b0};
wire [13:0] iaqaX = iaqa[13:0];

sdo1    i_DA0  (.c(clk2x),.d(iaqaX[ 0]),  .q(DA[ 0]));
sdo1    i_DA1  (.c(clk2x),.d(iaqaX[ 1]),  .q(DA[ 1]));
sdo1    i_DA2  (.c(clk2x),.d(iaqaX[ 2]),  .q(DA[ 2]));
sdo1    i_DA3  (.c(clk2x),.d(iaqaX[ 3]),  .q(DA[ 3]));
sdo1    i_DA4  (.c(clk2x),.d(iaqaX[ 4]),  .q(DA[ 4]));
sdo1    i_DA5  (.c(clk2x),.d(iaqaX[ 5]),  .q(DA[ 5]));
sdo1    i_DA6  (.c(clk2x),.d(iaqaX[ 6]),  .q(DA[ 6]));
sdo1    i_DA7  (.c(clk2x),.d(iaqaX[ 7]),  .q(DA[ 7]));
sdo1    i_DA8  (.c(clk2x),.d(iaqaX[ 8]),  .q(DA[ 8]));
sdo1    i_DA9  (.c(clk2x),.d(iaqaX[ 9]),  .q(DA[ 9]));
sdo1    i_DA10 (.c(clk2x),.d(iaqaX[10]),  .q(DA[10]));
sdo1    i_DA11 (.c(clk2x),.d(iaqaX[11]),  .q(DA[11]));
sdo1    i_DA12 (.c(clk2x),.d(iaqaX[12]),  .q(DA[12]));
sdo1    i_DA13 (.c(clk2x),.d(iaqaX[13]),  .q(DA[13]));



// assign PXD[9:0] = ienapwr?DA[13:4]:test_cntr[9:0];

// Dummy stuff to keep pins ==============================

//change to shutter control
	
//    assign GP[1:0]={seq_frame,seq_expos};
//    assign GP[7:0]=8'b0;
//   wire [7:0] mshutter;
//    assign GP[7:0]={3'h0, iwrt2,iwrt1,ild,seq_frame,seq_expos};
//    assign GP[7:0]=mshutter;
// pwr_mode[1] - enable shutter power (all "1" GP[i] - disable)
      OBUF  i_GP0  (.I(mshutter[0] || !pwr_mode[1]),  .O(GP[0]));
      OBUF  i_GP1  (.I(mshutter[1] || !pwr_mode[1]),  .O(GP[1]));
      OBUF  i_GP2  (.I(mshutter[2] || !pwr_mode[1]),  .O(GP[2]));
      OBUF  i_GP3  (.I(mshutter[3] || !pwr_mode[1]),  .O(GP[3]));
      OBUF  i_GP4  (.I(mshutter[4] || !pwr_mode[1]),  .O(GP[4]));
      OBUF  i_GP5  (.I(mshutter[5] || !pwr_mode[1]),  .O(GP[5]));
      OBUF  i_GP6  (.I(mshutter[6] || !pwr_mode[1]),  .O(GP[6]));
      OBUF  i_GP7  (.I(mshutter[7] || !pwr_mode[1]),  .O(GP[7]));

// keep ARO, ARST, MRST

  IBUF   i_ARO  (.I(ARO),  .O(iaro));
  IBUF   i_ARST (.I(ARST), .O(iarst));
  IBUF   i_MRST (.I(MRST), .O(imrst));

  IOBUF    i_ext_sync  (.I(1'b0),    .T(!sync_out),  .O(ext_in), .IO(EXT_SYNC));



endmodule
//
