//multisensor.h
#define I2C359_INC                    4 // slave adderss increment between sensors in 10359A board
#define I2C359_SLAVEADDR           0x10 // slave adderss of the 10359A board (move?)
#define I2C359_VERSION             0x00 // 32-bit register
#define I2C359_MINVERSION             0x0359104b
#define I2C359_DCM_SYSTEM          0x01
#define I2C359_DCM_SDRAM           0x02
#define I2C359_DCM_SENSOR1         0x03
#define I2C359_DCM_SENSOR2         0x04
#define I2C359_DCM_SENSOR3         0x05
  #define I2C359_DCM_INC             0x01
  #define I2C359_DCM_DEC             0x02
  #define I2C359_DCM_RESET           0x03
  #define I2C359_DCM_INC90           0x04
  #define I2C359_DCM_DEC90           0x08
  #define I2C359_DCM_RESET90         0x0c
  #define I2C359_DCM_HACT_INC        0x10
  #define I2C359_DCM_HACT_DEC        0x20
  #define I2C359_DCM_HACT_RESET      0x30
#define I2C359_CHNSEQ              0x06
  #define I2C359_SEQ(x,y,z) (((x)&3) | (((y)&3)<<2)  | (((z)&3)<<4)) // x - direct channel (no memory), y - second, z - third. 0- disabled, 1..3 channels (J2,J3,J4)



//#define I2C359_I2CMUX              0x07
  #define I2C359_I2CMUX_2MEM          0x1
  #define I2C359_I2CMUX_2SENSORS     0x0
#define I2C359_CLKSRC              0x08
  #define I2C359_CLKSRC_SYSTEM        0x00
  #define I2C359_CLKSRC_LOCAL         0x01
#define I2C359_MODE                0x09
#define I2C359_MODE_RESET             0x01 // 1 - reset, 0 - normal operation (persistent)
#define I2C359_MODE_DISABLE           0x02 // 1 - output disable reset, 0 - normal operation
#define I2C359_MODE_MUTI_EN           0x04 // 1 - enable multisensor mode, 0 - single sensor
#define I2C359_MODE_TEST_PATTERN      0x20 // 1 - enable test pattern from 10359, 0 - normal mode, sensor data

#define I2C359_MODE_INDIVIDUAL        0x10 // 0 - single frame sync for all channels, 1 - each sensor frame with individual frame sync
#define I2C359_MODE_PATTERN           0x20 // 1 - test pattern, 0 - sensor image

#define I2C359_HACT_WIDTH             0x0a // HACT duration when internally generated (I2C359_HACT_MODE==1), default 2596

#define I2C359_INTERFRAME_DELAY       0x0c // dealy between frames output, in lines. 16 bits . Needed in separate frames mode, large frames, non jp4
//#define I2C359_HACT_DLY               0x0d // bit 0 - delay HACT in frame 3 by 1 clock cycle, bit 1 - delay HACT in frame 2 by 1 c.c (to compensate for Bayer during hor. flips) 
#define I2C359_HACT_DLY               0x0d // bit 0 - delay HACT in frame 2 by 1 clock cycle, bit 1 - delay HACT in frame 3 by 1 c.c (to compensate for Bayer during hor. flips) 
#define I2C359_HACT_MODE              0x0e // 0 - use sensor HACT, 1 - use leading edge from the sensor, generate duration. Bit 0 sensor 0, 1 - 1, 2 - 2
#define I2C359_WIDTH2                 0x13 // line length  for memory channel 1 (2-nd frame in sequence). 16 bit (no I2C359_MSW), default 2596
#define I2C359_HEIGHT2                0x14 // frame height for memory channel 1 (2-nd frame in sequence). 16 bit (no I2C359_MSW), default 1940
#define I2C359_HBLANK2                0x15 // horizontal blank for memory channel 1 (2-nd frame in sequence). 16 bit (no I2C359_MSW), default 256
#define I2C359_VBLANK2                0x16 // vertical blank (before active frame) for memory channel 1 (2-nd frame in sequence). 16 bit (no I2C359_MSW), default 0
#define I2C359_WIDTH3                 0x23 // line length  for memory channel 3 (3-rd frame in sequence). 16 bit (no I2C359_MSW), default 2596
#define I2C359_HEIGHT3                0x24 // frame height for memory channel 3 (3-rd frame in sequence). 16 bit (no I2C359_MSW), default 1940
#define I2C359_HBLANK3                0x25 // horizontal blank for memory channel 3 (3-rd frame in sequence). 16 bit (no I2C359_MSW), default 256
#define I2C359_VBLANK3                0x26 // vertical blank (before active frame) for memory channel 3 (3-rd frame in sequence). 16 bit (no I2C359_MSW), default 0



#define I2C359_SDRAM_CHEN             0x40 // LSW of 32-bit, should be sent after I2C359_MSW, will be applied together
  #define I2C359_SDRAM_RUN(x)   (2 << (x <<1))  // enable=1, init=0
  #define I2C359_SDRAM_STOP(x)  (1 << (x <<1))  // enable=0, init=1
  #define I2C359_SDRAM_PAUSE(x) (3 << (x <<1))  // enable=0, init=0
#define I2C359_SDRAM_MANCMD           0x41 // LSW of 32-bit, should be sent after I2C359_MSW, will be applied together
#define I2C359_SDRAM_MAGIC43          0x43 // Undocumented

#define I2C359_SDRAM_START0           0x44 // SDRAM channel 0 start page number (each page 4096 pixels)
#define I2C359_SDRAM_START1           0x45 // SDRAM channel 1 start page number (each page 4096 pixels)
#define I2C359_SDRAM_START2           0x46 // SDRAM channel 2 start page number (each page 4096 pixels)
#define I2C359_SDRAM_START3           0x47 // SDRAM channel 3 start page number (each page 4096 pixels)

#define I2C359_SDRAM_CONF01A          0x4c // SDRAM channels 0,1 configuration A (with I2C359_MSW)
#define I2C359_SDRAM_CONF23A          0x4d // SDRAM channels 2,3 configuration A (with I2C359_MSW)
#define I2C359_SDRAM_CONF01B          0x4e // SDRAM channels 0,1 configuration B (with I2C359_MSW)
#define I2C359_SDRAM_CONF23B          0x4f // SDRAM channels 2,3 configuration B (with I2C359_MSW)


#define I2C359_MSW                 0x50 // write 16 MSBs of the next 32 bit word. Will be combined with other 16-bit writes
#define I2C359_SDRAM_PAGE_WR       0x63 // any data to start 64-word page write to buffer
#define I2C359_SDRAM_PAGE_RD       0x64 // any data to start 64-word page read to buffer
#define I2C359_SDRAM_DATA          0x70 // 16-bit data to be writtent to channel 4 or read from channel5

#define I2C359_DCM_STATUS          0x0F // readonly, status of DCMs 
#define I2C359_DCM_OFL(x,status)    ((status) & (1<<((((x) & 3 ) <<2) +0))) // 0 - SDRAM, 1..3 - sensors (status - data read from I2C359_DCM_STATUS)
#define I2C359_DCM_DONE(x,status)   ((status) & (1<<((((x) & 3 ) <<2) +1))) // 0 - SDRAM, 1..3 - sensors
#define I2C359_DCM_LOCKED(x,status) ((status) & (1<<((((x) & 3 ) <<2) +2))) // 0 - SDRAM, 1..3 - sensors
  #define I2C359_DCM_ERR_UNKNOWN  2
  #define I2C359_DCM_ERR_OVFL     3
  #define I2C359_DCM_ERR_NODONE   4
  #define I2C359_DCM_ERR_NOLOCKED 5

/*!
CHEN_LOW,CHEN_HIGH - control 8 SDRAM channels + refresh (in CHEN_HIGH)
each dibit:
 0 - nop
 1 - disable
 2 - enable, init
 3 - disable, init
 MANCMD_HIGH,MANCMD_LOW - data to be directly applied to SDRAM for 1 clock cycle
*/



#define I2C359_CLK_NUMBER         4 // system clock number for the 10359A on-board clock generator

int multisensor_pgm_detectsensor   (struct sensor_t * sensor,  struct framepars_t * thispars, struct framepars_t * prevpars, int frame8);
//int multisensor_pgm_afterinit0  (struct sensor_t * sensor,  struct framepars_t * thispars, struct framepars_t * prevpars, int frame8);
//int multisensor_pgm_window_common0 (struct sensor_t * sensor,  struct framepars_t * thispars, struct framepars_t * prevpars, int frame8);


//#define FRAMEPAR_MODIFIED(x) (thispars->mod[(x) >> 5] & (1<<((x) & 0x1f)))
//#define SETFRAMEPARS_SET(p,v)        { pars_to_update[nupdate  ].num= (p) ;  pars_to_update[nupdate++].val=(v);}
//#define SETFRAMEPARS_UPDATE(p)       { pars_to_update[nupdate  ].num= (p) ;  pars_to_update[nupdate++].val=thispars->pars[(p) & 0xffff];}
//#define SETFRAMEPARS_UPDATE_SET(p,v) { pars_to_update[nupdate  ].num= (p) ;  pars_to_update[nupdate++].val=thispars->pars[(p) & 0xffff]?thispars->pars[(p) & 0xffff]:(v);}
//#define SETFRAMEPARS_COND(p,v)  { if (unlikely((v)!=thispars->pars[p])) { pars_to_update[nupdate  ].num= (p) ;  pars_to_update[nupdate++].val=(v);} }

///set parameter for the sensor register and hardware itself
/// f - fpga address, s - i2c slave address, r - sensor register, v - value to set
/*
  struct frameparspair_t  pars_to_update[8];
  int nupdate=0;
...
   if (nupdate)  setFramePars(thispars, nupdate, pars_to_update);  /// save changes to sensor register shadows

*/
