#!/usr/bin/php -q
<?php
/*!***************************************************************************
*! FILE NAME   : eyesis_auto_launch.php
*! DESCRIPTION : The script reads the time from the 1st Eyesis camera and sets 
*!               a 10 seconds delay before it starts camogms on all the 3 cameras.
*! AUTHOR      : Oleg K Dzhimiev (oleg@elphel.com)
*! Copyright (C) 2010 Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!  $Log: eyesis_auto_launch.php,v $
*!  Revision 1.1  2010/09/01 01:17:37  dzhimiev
*!  1. removed elphel-eyesis-gui - cause it's a separate package     now.
*!  2. eyesis_auto_launch.php - a script for eyesis autostart recording after reboot.
*!
*/

$app_mode_file="/var/state/APPLICATION_MODE";

$rec_delay=10; //seconds

if (is_file($app_mode_file)){
  $isis_app_mode = file_get_contents($app_mode_file);
  printf("The application mode is $isis_app_mode\n");

  $ip[0]=get_my_ip();    printf("My IP is $ip[0]\n");
  $ip[1]=ip_inc($ip[0]); //printf("IP2   is $ip[1]\n");
  $ip[2]=ip_inc($ip[1]); //printf("IP3   is $ip[2]\n");

  //launch camogm on the current camera
  camogm_launch($ip[0]);

  // sync recording start part
  if ($isis_app_mode==1) {
    // get time from master camera
    if ($fp = fopen("http://".$ip[0]."/camogmgui/camogm_interface.php?cmd=gettime", 'r')) {
      $content = '';
      while ($line = fread($fp, 1024)) {
      $content .= $line;
      }
      $xml = new SimpleXMLElement($content);
      $current_time=$xml -> gettime;
    } else {
      // just PC time
      $pc_time=getdate();
      $current_time = $pc_time[0];
    }

    $start_timestamp=$current_time+$rec_delay;

    for ($i=0;$i<count($ip);$i++) fopen("http://".$ip[$i]."/camogmgui/camogm_interface.php?cmd=set_start_after_timestamp&start_after_timestamp=$start_timestamp", 'r');

    // start recording command for all
    for ($i=0;$i<count($ip);$i++) camogm_rec_start($ip[$i]);
  }

}else{
  printf("A regular camera\n");
  $ip[0]=get_my_ip();    printf("My IP is $ip[0]\n"); 
  exit(0);
}

function get_my_ip(){
  //return $_SERVER['SERVER_ADDR'];
  exec("ifconfig",$ifconfig);
  $ifconfig_cut=strstr($ifconfig[1],"addr:");
  $ip=substr($ifconfig_cut,5,strpos($ifconfig_cut," ")-5);
  return $ip;
}

function ip_inc($ip){
  $var=substr(strrchr($ip,"."),1);
  $new_ip=substr($ip,0,strpos($ip,$var)).($var+1);
  return $new_ip;
}

function camogm_launch($ip){

  //enable JP4 mode -- don't forget to comment this out later
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=set_parameter&pname=COLOR&pvalue=5", 'r');

  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=run_camogm&debug=/var/html/camogm.log&debuglev=0", 'r');

  // set "/var/0" prefix
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=set_debuglev&debuglev=0", 'r');
  // set "/var/0" prefix
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=set_prefix&prefix=/var/0/", 'r');

  // mount hdd //!!!!!!!!!!!!! /dev/hda4 is extended and contains hda5-hda9!!!!!!!!!!!!!!!!!
  // mount primary partitions
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=mount&partition=/dev/hda1&mountpoint=/var/0", 'r');
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=mount&partition=/dev/hda2&mountpoint=/var/1", 'r');
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=mount&partition=/dev/hda3&mountpoint=/var/2", 'r');
  // mount partitions inside the extended partition
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=mount&partition=/dev/hda5&mountpoint=/var/3", 'r');
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=mount&partition=/dev/hda6&mountpoint=/var/4", 'r');
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=mount&partition=/dev/hda7&mountpoint=/var/5", 'r');
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=mount&partition=/dev/hda8&mountpoint=/var/6", 'r');
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=mount&partition=/dev/hda9&mountpoint=/var/7", 'r');
  // set .mov format
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=setmov", 'r');
  // set frames_per_chunk in exif to 1
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=set_frames_per_chunk&frames_per_chunk=1", 'r');
  // set default split parameters
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=set_duration&duration=100", 'r');
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=set_size&size=600000000", 'r');
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=set_max_frames&max_frames=100", 'r');
  
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=set_prefix&prefix=/var/0/", 'r');
}

function camogm_rec_start($ip){
  fopen("http://".$ip."/camogmgui/camogm_interface.php?cmd=start", 'r');
}

?>