<?php
/*!*******************************************************************************
*! FILE NAME  : 10364.php
*! DESCRIPTION: initializes and controls motors connected to 10364 board
*! Copyright (C) 2010 Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*/
$SYSTEM_CLOCK=160000000/2/12; /// sequencer period
$STEPS_PER_REVOLUTION=0xe00;
$CONFIG_ADDR =0x70;
$CONFIG_DATA= 0xba<<24;
$BASE_ADDR   =0x7c;
$MAX_SPEED=   4000.0 ;  ///encoder pulses/sec at full speed
$MAX_ACCEL=   500.0 ;   /// acceleration in full speed/second
$GAIN=        0.00002;  /// TODO: define units - additional PWM signal for speed difference
$FREQ=        45000;    /// sample frequency for period measurement
//$BRAKE_SPEED=5;        /// if position error==0 and |speed| <+/- 5 counts of encoded period (from maximal) - brake motors
$BRAKE_SPEED= 32;       /// if position error==0 and |speed| <+/- 5 counts of encoded period (from maximal) - brake motors (32 - always)
$PUSH=        1.0;      /// "push" power applied when the motor is stopped, in ratio to full. Overwrites other data if higher
$REVERSE=     false;    /// opposite relation between Encoders and Motors
$TABLES=      false;    /// show speed tables
$READMACK=    false;
$INIT=        false;    /// initialize 10364
$RESET=       false;    /// reset motors positions to zeroes
unset ($M1);            /// target position of motor1
unset ($M2);            /// target position of motor2
unset ($M3);            /// target position of motor3
$PWM_CYCLE=         67; /// 0x43
$PWM_DELAY=          2; ///
$DEGLITCH_DIV=       1; ///
$DECR_ON_PULSE=      4;
$DISABLE=            false;
$ENABLE=             false;
$INSERT=             0; /// repeat 60 degree rotations fro inserting allen keys
$SLEEP=              0; //sleep time before action
unset($WAIT);
//$ENC_PERIOD_CYCLE=1111;
//MAX_SPEED=4000&MAX_ACCEL=3000&FREQ=45000&GAIN=0.00002&DECR_ON_PULSE=4&INIT

foreach ($_GET as $key=>$value) {
  switch (strtoupper($key)) {
    case 'MAX_SPEED':       $MAX_SPEED=$value+0.0;break;
    case 'MAX_ACCEL':       $MAX_ACCEL=$value+0.0;break;
    case 'GAIN':            $GAIN=$value+0.0;break;
    case 'FREQ':            $FREQ=$value+0.0;break;
    case 'BRAKE_SPEED':     $BRAKE_SPEED=$value+0.0;break;
    case 'PUSH':            $PUSH=$value+0.0;break;
    case 'REVERSE':         $REVERSE=true;break;
    case 'TABLES':          $TABLES=true;break;
    case 'READBACK':        $READBACK=true;break;
    case 'INIT':            $INIT=true;break;
    case 'RESET':           $RESET=true;break;
    case 'DISABLE':         $DISABLE=true;break;
    case 'ENABLE':          $ENABLE=true;break;
    case 'M1':              $M1=$value+0;break;
    case 'M2':              $M2=$value+0;break;
    case 'M3':              $M3=$value+0;break;
    case 'INSERT':          $INSERT=$value+0;break;
    case 'WAIT':            $WAIT=$value+0;break;
    case 'PWM_CYCLE':       $PWM_CYCLE=$value+0;break;
    case 'PWM_DELAY':       $PWM_DELAY=$value+0;break;
    case 'DEGLITCH_DIV':    $DEGLITCH_DIV=$value+0;break;
    case 'DECR_ON_PULSE':   $DECR_ON_PULSE=$value+0;break;
    case 'SLEEP':           $SLEEP=$value+0;break;

//    case 'ENC_PERIOD_CYCLE':$ENC_PERIOD_CYCLE=$value+0;break; /// Calculate from FREQ?
  }
}
$ENC_PERIOD_CYCLE=(int) floor($SYSTEM_CLOCK/$FREQ);

/*
if ($INIT) {
  echo "<pre>\n";
  printf ("ENC_PERIOD_CYCLE=0x%x (%d)\n",$ENC_PERIOD_CYCLE,$ENC_PERIOD_CYCLE);
  echo "</pre>\n";
}
*/

sleep($SLEEP);

if ($TABLES || $INIT) {
  $pwm=calcMotorTable($MAX_SPEED,$MAX_ACCEL,$FREQ,$GAIN,$PUSH,$BRAKE_SPEED,$REVERSE);

}


if ($TABLES) {
  printTable1($pwm,$FREQ);
  echo '<hr/>';
  printTable2($pwm);
  echo '<hr/>';
  printTable3($pwm);
  exit (0); /// no hardware if 'tables'
}
$fpga_revision=elphel_fpga_read(0x13);
if ($fpga_revision<0x3534012) {
  echo "<h2>This program required FPGA revision >= 0x3534012, exiting </h2>";
  exit (1);
}

if ( exec('fpcf -req_io')==1) { 
  echo "<h2>10369 I/O board detected, it is not compatible with this program<br/>designed for the 10364 motor driver board, exiting </h2>";
  exit (1);
}


//echo '$INIT='.$INIT;
if ($INIT) {

  elphel_fpga_write($CONFIG_ADDR,$CONFIG_DATA); /// set FPGA to use 10364 board and disable manual commands (i.e. i2c)
  elphel_fpga_write($BASE_ADDR+1,4);            /// point to control register
//  elphel_fpga_write($BASE_ADDR+0,6);            /// disable motors, reset positions
  elphel_fpga_write($BASE_ADDR+0,0);            /// disable motors, do not reset positions

  if (!($PWM_CYCLE>0)        || !($PWM_CYCLE<256))         $PWM_CYCLE=     67; /// 0x43
  if (!($PWM_DELAY>0)        || !($PWM_DELAY<8))           $PWM_DELAY=      2; ///
  if (!($DEGLITCH_DIV>0)     || !($DEGLITCH_DIV<16))       $DEGLITCH_DIV=   7; /// 0x457
  if (!($ENC_PERIOD_CYCLE>0) || !($ENC_PERIOD_CYCLE<4096)) $ENC_PERIOD_CYCLE=1111;
  if (!($DECR_ON_PULSE>=0)   || !($DECR_ON_PULSE<32))      $DECR_ON_PULSE=2;

  elphel_fpga_write($BASE_ADDR+0, ($PWM_CYCLE) | ($PWM_DELAY<<8) | ($DEGLITCH_DIV<<12)); ///pwm_cycle=0x43, pwm_delay=2, deglitch_div=7
  elphel_fpga_write($BASE_ADDR+0, $ENC_PERIOD_CYCLE);                                    ///enc_period_cycle=0x457
  elphel_fpga_write($BASE_ADDR+0, $DECR_ON_PULSE);                                       ///decrement_on_pulse
/*
  elphel_fpga_write($BASE_ADDR+0, 1); // target positions
  elphel_fpga_write($BASE_ADDR+0, 0); // target position1
  elphel_fpga_write($BASE_ADDR+0, 0); // target position2
  elphel_fpga_write($BASE_ADDR+0, 0); // target position3
*/

  FPGAtable($pwm);
  elphel_fpga_write($BASE_ADDR+1,4);            /// point to control register
//  elphel_fpga_write($BASE_ADDR+0,7);            /// reset, then enable motors
  elphel_fpga_write($BASE_ADDR+0,1);            /// do not reset, just then enable motors
}


if ($RESET) {
  elphel_fpga_write($BASE_ADDR+1,4);            /// point to control register
  elphel_fpga_write($BASE_ADDR+0,6);            /// disable motors, reset positions
  elphel_fpga_write($BASE_ADDR+1, 1); // target positions
  elphel_fpga_write($BASE_ADDR+0, 0); // target position1
  elphel_fpga_write($BASE_ADDR+0, 0); // target position2
  elphel_fpga_write($BASE_ADDR+0, 0); // target position3
  elphel_fpga_write($BASE_ADDR+1,4);            /// point to control register
  if (!$DISABLE)  elphel_fpga_write($BASE_ADDR+0,7);  /// reset, then enable motors (if not specifically disabled)
}

///Read positions before motros started to move
$positions=readPositions();

if ($ENABLE) {
  elphel_fpga_write($BASE_ADDR+1,4);  /// point to control register
  elphel_fpga_write($BASE_ADDR+0,3);  /// enable motors
}

if ($DISABLE) {
  elphel_fpga_write($BASE_ADDR+1,4);  /// point to control register
  elphel_fpga_write($BASE_ADDR+0,2);  /// disable motors
}

if (isset($M1)) {
  $M1&=0xffff;
  elphel_fpga_write($BASE_ADDR+1,  1);            /// point to motor 1 position
  elphel_fpga_write($BASE_ADDR+0,$M1);            /// set motor 1 target position
  if ($M1>0x8000) $M1-=0x10000;
}
if (isset($M2)) {
  $M2&=0xffff;
  elphel_fpga_write($BASE_ADDR+1,  2);            /// point to motor 2 position
  elphel_fpga_write($BASE_ADDR+0,$M2);            /// set motor 2 target position
  if ($M2>0x8000) $M2-=0x10000;
}
if (isset($M3)) {
  $M3&=0xffff;
  elphel_fpga_write($BASE_ADDR+1,  3);            /// point to motor 3 position
  elphel_fpga_write($BASE_ADDR+0,$M3);            /// set motor 3 target position
  if ($M3>0x8000) $M3-=0x10000;
}
if ($INSERT>0) {
  $rot=$STEPS_PER_REVOLUTION/6;
  $timeout=2;
  $tolerance=1;
  $rotPos=array();
  for ($i=1;$i<=3;$i++) $rotPos[$i]=$positions[$i]+$rot;
  for ($i=0;$i<$INSERT;$i++) {
//$positions[1]
    elphel_fpga_write($BASE_ADDR+1,  1);          /// point to motor 1 position
    elphel_fpga_write($BASE_ADDR+0, $rotPos[1]);  /// set motor 1 target position
    elphel_fpga_write($BASE_ADDR+0, $rotPos[2]);  /// set motor 2 target position
    elphel_fpga_write($BASE_ADDR+0, $rotPos[3]);  /// set motor 3 target position
    waitPositions($rotPos,$tolerance,$timeout);
    elphel_fpga_write($BASE_ADDR+1,  1);          /// point to motor 1 position
    elphel_fpga_write($BASE_ADDR+0, $positions[1]);  /// set motor 1 target position
    elphel_fpga_write($BASE_ADDR+0, $positions[2]);  /// set motor 2 target position
    elphel_fpga_write($BASE_ADDR+0, $positions[3]);  /// set motor 3 target position
    waitPositions($positions,$tolerance,$timeout);
  }
}
///TODO: Verify all motors finished before exiting (add tolerance?)
if (isset ($WAIT)) {
  $waitPos=array();
  if (isset($M1)) $waitPos[1]=$M1;
  if (isset($M2)) $waitPos[2]=$M2;
  if (isset($M3)) $waitPos[3]=$M3;
  if (count($waitPos)>0) {
    if ($WAIT>0) $timeout=$WAIT;
    else $timeout=20;
    $tolerance=1;
    waitPositions($waitPos,$tolerance,$timeout);
    $positions=readPositions();
  }
}

if ($READBACK) {
  $rtable=FPGAreadbackTable();
  printTable1($rtable,$FREQ);
  echo '<hr/>';
  printTable2($rtable);
  echo '<hr/>';
  printTable3($rtable);
  exit (0);
}

    $xml = new SimpleXMLElement("<?xml version='1.0'?><motors/>");
    $xml->addChild ('motor1',$positions[1]);
    $xml->addChild ('motor2',$positions[2]);
    $xml->addChild ('motor3',$positions[3]);
    $rslt=$xml->asXML();
    header("Content-Type: text/xml");
    header("Content-Length: ".strlen($rslt)."\n");
    header("Pragma: no-cache\n");
    printf($rslt);


exit(0);


function readPositions() {
  global $BASE_ADDR;
  $positions=array();
  elphel_fpga_write($BASE_ADDR+1,  1);           /// point to motor 1 position
  $positions[1]=elphel_fpga_read($BASE_ADDR+0);  /// read motor 1 current position
  elphel_fpga_write($BASE_ADDR+1,  2);           /// point to motor 2 position
  $positions[2]=elphel_fpga_read($BASE_ADDR+0);  /// read motor 2 current position
  elphel_fpga_write($BASE_ADDR+1,  3);           /// point to motor 3 position
  $positions[3]=elphel_fpga_read($BASE_ADDR+0);  /// read motor 3 current position
  for ($i=1;$i<4;$i++) {
    if ($positions[$i]>0x8000) $positions[$i]-=0x10000;
  }
  return $positions;
}

function waitPositions($targetPositions,$tolerance=1,$timeout=10) {
  $bailout=time()+$timeout;
  global $BASE_ADDR;
  $positions=array();
/*
echo "<!--\n";
var_dump($targetPositions);echo"\n";
var_dump($tolerance);echo"\n";
var_dump($timeout);echo"\n";
    elphel_fpga_write($BASE_ADDR+1,  1);           /// point to motor 1 position
    $positions[1]=elphel_fpga_read($BASE_ADDR+0);  /// read motor 1 current position
    elphel_fpga_write($BASE_ADDR+1,  2);           /// point to motor 2 position
    $positions[2]=elphel_fpga_read($BASE_ADDR+0);  /// read motor 2 current position
    elphel_fpga_write($BASE_ADDR+1,  3);           /// point to motor 3 position
    $positions[3]=elphel_fpga_read($BASE_ADDR+0);  /// read motor 3 current position
    for ($i=1;$i<4;$i++) {
      if ($positions[$i]>0x8000) $positions[$i]-=0x10000;
    }

var_dump($positions);echo"\n";
echo "-->\n";
*/

  while (time()<$bailout) {
    elphel_fpga_write($BASE_ADDR+1,  1);           /// point to motor 1 position
    $positions[1]=elphel_fpga_read($BASE_ADDR+0);  /// read motor 1 current position
    elphel_fpga_write($BASE_ADDR+1,  2);           /// point to motor 2 position
    $positions[2]=elphel_fpga_read($BASE_ADDR+0);  /// read motor 2 current position
    elphel_fpga_write($BASE_ADDR+1,  3);           /// point to motor 3 position
    $positions[3]=elphel_fpga_read($BASE_ADDR+0);  /// read motor 3 current position
    for ($i=1;$i<4;$i++) {
      if ($positions[$i]>0x8000) $positions[$i]-=0x10000;
    }
    if ((!isset($targetPositions[1]) || (abs($positions[1]-$targetPositions[1])<=$tolerance)) &&
        (!isset($targetPositions[2]) || (abs($positions[2]-$targetPositions[2])<=$tolerance)) &&
        (!isset($targetPositions[3]) || (abs($positions[3]-$targetPositions[3])<=$tolerance))) return true;
  }
  return false;
}



function FPGAtable($pwm) {
  global $BASE_ADDR;
  elphel_fpga_write($BASE_ADDR+1,512);
  for ($i=0; $i<64; $i++) {
    for ($j=0;$j<64;$j+=8) {
      $v32=0;
      for ($k=0;$k<8;$k++) {
        $v=$pwm[$i][$j+$k];
        if ($v<0) $v=8+abs($v);
        $v32+=$v << (4*($k));
      }
      elphel_fpga_write($BASE_ADDR+0,$v32);
    }
  }
}


function FPGAreadbackTable() {
  global $BASE_ADDR;
  $pwm=array();
  $d=array();
  for ($n=0; $n<512; $n++) {
    elphel_fpga_write($BASE_ADDR+1,512+$n);
    $d[0]=elphel_fpga_read($BASE_ADDR);
    elphel_fpga_write($BASE_ADDR+1,512+$n+1024);
    $d[1]=elphel_fpga_read($BASE_ADDR);
    $i= $n>>3;
    $j0= ($n & 7)<<3;
    if ($j0==0) $pwm[$i]=array();
    for ($k=0;$k<8;$k++) {
      $pwm[$i][$j0+$k]=($d[$k>>2]>>(4*($k & 3)))&0xf;
      if ($pwm[$i][$j0+$k]>8) $pwm[$i][$j0+$k]=(8-$pwm[$i][$j0+$k]);
    }
  }
/*
  echo '<pre>';
  var_dump($pwm);
  echo '</pre>';
*/
  return $pwm;

}











function printTable1($pwm,$FREQ) {
  echo "<table>\n";
  echo "<tr>\n";
  echo '<td>\\code</td>';
  for ($vcc=-31;$vcc<31;$vcc++) {
    $vc=($vcc<0)?(-32-$vcc):(31-$vcc);  
    echo "<td>".$vc."</td>";
  }
  echo "</tr>\n";

  echo "<tr>\n";
  echo '<td>\\speed</td>';
  for ($vcc=-31;$vcc<31;$vcc++) {
    $vc=($vcc<0)?(-32-$vcc):(31-$vcc);  
    $v=speed($FREQ,abs($vc));
    if ($vc<0) $v=-$v;
    echo "<td>".round($v)."</td>";
  }
  echo "</tr>\n";

  for ($s=-31; $s<32; $s++) {
    echo "<tr><td>$s</td>";
    $i=($s<0)?(32-$s):$s;
    for ($vcc=-31;$vcc<31;$vcc++) {
      $vc=($vcc<0)?(-32-$vcc):(31-$vcc);  
      $j=($vc<0)?(32-$vc):$vc;
      $v=$pwm[$i][$j];
      if      ($v==8) $v='*B';
      else if ($v==-7) $v='-';
      else if ($v==7) $v='+';
      else if ($v>0)  $v='+'.$v;
      echo '<td>';
      if (($i==0) || ($vc==31) || ($vc==-31)) echo '<b>';
      echo $v;
      if (($i==0) || ($vc==31) || ($vc==-31)) echo '</b>';
      echo '</td>';
    }
    echo "</tr>\n";
  }
  echo "</table>\n";
}

function printTable2($pwm) {
  echo "<h2>Raw data</h2>";

  echo "<table>\n";
  for ($i=0; $i<64; $i++) {
    echo "<tr>";
    for ($j=0;$j<64;$j++) {
      $v=$pwm[$i][$j];
      if ($v<0) $v=8+abs($v);
      echo '<td>';
        if (($i==0) || ($i==32) || ($j==32)) echo '<b>';
        printf('%x',$v);
        if (($i==0) || ($i==32) || ($j==32)) echo '</b>';
      echo '</td>';
    }
    echo "</tr>\n";
  }
  echo "</table>\n";
}
function printTable3($pwm) {
  echo "<h2>FPGA simulation data for the motor table</h2>";
  echo "<pre>\n";
  for ($i=0; $i<64; $i++) {
    for ($j=0;$j<64;$j+=8) {
      $v32=0;
      for ($k=0;$k<8;$k++) {
        $v=$pwm[$i][$j+$k];
        if ($v<0) $v=8+abs($v);
        $v32+=$v << (4*($k));
      }
//      printf('%08x (%d)',$v32,$v32);
      printf('%08x ',$v32);
    }
    echo "\n";
  }
  echo "</pre>\n";
}






function calcMotorTable($MAX_SPEED,$MAX_ACCEL,$FREQ,$GAIN,$PUSH,$BRAKE_SPEED,$REVERSE){

  $a=$MAX_SPEED*$MAX_ACCEL;
  $maxs=63;
  $speeds=array();
  for ($iv=0;$iv<64;$iv++) {
    $speeds[$iv]=($iv<32)? speed($FREQ,$iv): (-speed($FREQ,$iv & 0x1f));
  }
/// Create 64x64 array
  $pwm=array();
  for ($is=0;$is<64;$is++) {
    $pwm[$is]=array();
    $s=($is<32)?$is:(-($is&0x1f));
    $av0=sqrt($a*abs($s)/2);
    $v0=($s>=0)?-$av0:$av0;
    for ($iv=0;$iv<64;$iv++) {
      $v=$speeds[$iv];
      $diff=($v0-$v);
      $p=$v0/$MAX_SPEED+$GAIN*$diff;
      if ($p>1) $p=1;
      else if ($p<-1) $p=-1;
      $p=round(8*$p);
      if ($p>1) $p--;
      if ($p<-1) $p++;
      if ($p==0) $p=0;
      if (($PUSH >0) && (($iv & 0x1f)== 0x1f) && ($s!=0)) { //lowest speed/stop only
        $p_stop=round(7*$PUSH);
        if      (($s<0) && ($p_stop>$p)) $p=$p_stop;
        else if (($s>0) && ($p_stop>(-$p))) $p=-$p_stop;
      }
      if (($s==0) && (($iv & 0x1f) > (31-$BRAKE_SPEED))) $p=8;
      $pwm[$is][$iv]=$REVERSE?(($p==8)?$p:-$p):$p;
    }
  }
  return $pwm;
}

function speed($f,$value) {
   $m=$value & 0x3;
   switch ($value >>2) {
     case 0: $v=     $m; break;
     case 1: $v=  4+ $m; break;
     case 2: $v=  8+ ($m<<1); break;
     case 3: $v= 16+ ($m<<2); break;
     case 4: $v= 32+ ($m<<3); break;
     case 5: $v= 64+ ($m<<4); break;
     case 6: $v=128+ ($m<<5); break;
     case 7: $v=256+ ($m<<6); break;
   }
   if ($v==0) $v=1;
   return $f/$v;
}
?> 
