#!/usr/local/sbin/php -q
<?php
/*!
*! PHP script
*! FILE NAME  : bdetect.php
*! DESCRIPTION: This programs tries to recognize the "sensor" board FPGA using JTAG
*!              and control of the main FPGA of the SCL and SDA pins.
*!              It changes values of these two pins and reads JTAG boundary scan
*!              of the 'sensor board' FPGA.
*! AUTHOR     : elphel, spectr, dzhimiev
*! Copyright (C) 2008 Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!
*!  $Log: bdetect.php,v $
*!  Revision 1.4  2009/07/29 15:51:07  elphel
*!  Cleanup, got rid of exec ("fpcf...), used php extension to interface FPGA
*!
*!  Revision 1.3  2009/02/05 19:57:21  dzhimiev
*!  + creates file "var/log/bdetect" that contains a detected extension board ID
*!
*!
*/

/*!
 * This programs tries to recognize the "sensor" board FPGA using JTAG
 * and control of the main FPGA of the SCL and SDA pins.
 * It changes values of these two pins and reads JTAG boundary scan
 * of the 'sensor board' FPGA
 
 */
$bdt_log = "/var/log/bdetect";

function readbscan($filename) {
   $jtag  = fopen($filename, 'r+');
   $ffs=  str_repeat ( chr(0xff), 97);
   $ret=  fwrite($jtag, $ffs);
//   printf ("fwrite returned %d<br/>\n",$ret);
   fseek ($jtag,0);
   $boundary= fread($jtag, 97);
   fclose($jtag);
  return $boundary;
}

function checkSclSda($boards) {
   $X313_I2C_CMD=0x5f;
   $X3X3_I2C_STOP_BITS=  0x2000;
   $X3X3_I2C_SCL_0_BITS= 0x10000;
   $X3X3_I2C_SCL_1_BITS= 0x20000;
   $X3X3_I2C_SCL_Z_BITS= 0x30000;
   $X3X3_I2C_SDA_0_BITS= 0x40000;
   $X3X3_I2C_SDA_1_BITS= 0x80000;
   $X3X3_I2C_SDA_Z_BITS= 0xc0000;

   elphel_fpga_write($X313_I2C_CMD,$X3X3_I2C_STOP_BITS) ; /// stop i2c hardware controller if it was on (will not be restored on exit)
   elphel_fpga_write($X313_I2C_CMD,$X3X3_I2C_SCL_0_BITS); /// SCL=0
   elphel_fpga_write($X313_I2C_CMD,$X3X3_I2C_SDA_0_BITS); /// SDA=0
   $test=array();
   $boundary=readbscan('/dev/sfpgabscan');
   foreach ($boards as $key => $value) {
     $test[$key]= (((ord($boundary[$value['sda'] >> 3]) >> (7 -($value['sda'] & 7))) &1) == 0) &&
                  (((ord($boundary[$value['scl'] >> 3]) >> (7 -($value['scl'] & 7))) &1) == 0);
   }
   elphel_fpga_write($X313_I2C_CMD,$X3X3_I2C_SCL_1_BITS); /// SCL=1

   $boundary=readbscan('/dev/sfpgabscan');
   foreach ($boards as $key => $value) {
     $test[$key] = $test[$key] &&
                     (((ord($boundary[$value['sda'] >> 3]) >> (7 -($value['sda'] & 7))) &1) == 0) &&
                     (((ord($boundary[$value['scl'] >> 3]) >> (7 -($value['scl'] & 7))) &1) == 1);
   }
   elphel_fpga_write($X313_I2C_CMD,$X3X3_I2C_SCL_0_BITS); /// SCL=0
   elphel_fpga_write($X313_I2C_CMD,$X3X3_I2C_SDA_1_BITS); /// SDA=1

   $boundary=readbscan('/dev/sfpgabscan');
   foreach ($boards as $key => $value) {
     $test[$key] = $test[$key] &&
                     (((ord($boundary[$value['sda'] >> 3]) >> (7 -($value['sda'] & 7))) &1) == 1) &&
                     (((ord($boundary[$value['scl'] >> 3]) >> (7 -($value['scl'] & 7))) &1) == 0);
   }
   elphel_fpga_write($X313_I2C_CMD,$X3X3_I2C_SCL_1_BITS); /// SCL=1

   $boundary=readbscan('/dev/sfpgabscan');
   elphel_fpga_write($X313_I2C_CMD,$X3X3_I2C_SCL_Z_BITS); /// SCL_EN=0
   elphel_fpga_write($X313_I2C_CMD,$X3X3_I2C_SDA_Z_BITS); /// SCL_EN=0
   
   foreach ($boards as $key => $value) {
     $test[$key] = $test[$key] &&
                     (((ord($boundary[$value['sda'] >> 3]) >> (7 -($value['sda'] & 7))) &1) == 1) &&
                     (((ord($boundary[$value['scl'] >> 3]) >> (7 -($value['scl'] & 7))) &1) == 1);
     if($test[$key]) {
       return $value['model'];
     }
   }

   return "";
}   
function save_log($log) {
	global $bdt_log;

	$file = fopen($bdt_log, "w+");
	if($file) {
		fwrite($file, $log);
		fclose($file);
	}
}

$boards=array (
                '0' => array ('model' => '10347', 'scl' =>241,'sda' => 199),  // E4, C1
                '1' => array ('model' => '10359', 'scl' =>280,'sda' => 296)   // H6, J5

);


$rslt=checkSclSda($boards);
echo $rslt;
save_log($rslt);
if ($rslt=="") exit (-1);
exit (0);
//echo "<pre>\n";
//var_dump ($rslt);
//echo "</pre>\n";

?> 
