<?php
/*!***************************************************************************
*! FILE NAME  : i2c.inc
*! DESCRIPTION: Provides functions to read/write over i2c buses in the camera,
*!              low-level R/W and additionally:
*!              Setting per-slave protection mask,
*!              Synchronizing system clock with a "CMOS" one
*!              Reading from/writing to EEPROM memory
*! Copyright (C) 2008 Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!  $Log: i2c.inc,v $
*!  Revision 1.2  2010/08/10 21:11:40  elphel
*!  added EEPROM support of the 10359 and 10338D boards.
*!
*!  Revision 1.1.1.1  2008/11/27 20:04:03  elphel
*!
*!
*!  Revision 1.3  2008/10/04 16:09:04  elphel
*!  updating FPGA time with system time from CMOS
*!
*!  Revision 1.2  2008/06/04 20:26:23  elphel
*!  includes SMBus operations, needed to activate USB on the 10369A board
*!
*!  Revision 1.2  2008/03/16 01:24:09  elphel
*!  Added i2c speed control interface
*!
*!  Revision 1.1  2008/03/15 23:05:18  elphel
*!  split i2c.php into i2c.php and i2c.inc (to be used from other scripts)
*!
*!
*/
$i2c_slow_safe=array(3,4,2,2,7,7); /// sensor i2c delays are 2,2,1,1,7,7

function i2c_ctl($bus,$data="") { //!$data - string of decimal values separated by ":", empty - don't change
   $size=6; // bytes per bus
   $data=trim($data);
   if ($data!="") {
     $darr=explode(":",$data);
     $i2cctl  = fopen('/dev/xi2cctl', 'w');
     for ($i=0; ($i<$size) && ($i<count($darr)); $i++) if ($darr[$i]!=""){
       fseek ($i2cctl, $bus*$size+$i) ;
       fwrite($i2cctl, chr($darr[$i]+0), 1);
     }
     fclose($i2cctl);
   }
   $i2cctl  = fopen('/dev/xi2cctl', 'r');
   fseek ($i2cctl, $bus*$size) ;
   $data = fread($i2cctl, $size);
   fclose($i2cctl);
   $xml = new SimpleXMLElement("<?xml version='1.0'?><i2cctl/>");
           $xml->addChild ('scl_high',ord($data[0]));
           $xml->addChild ('scl_low', ord($data[1]));
           $xml->addChild ('slave2master', ord($data[2]));
           $xml->addChild ('master2slave', ord($data[3]));
           $xml->addChild ('filter_sda',   ord($data[4]));
           $xml->addChild ('filter_scl',   ord($data[5]));
//           $data=$xml->asXML();
   return $xml;

} // end of i2c_ctl()

function i2c_ctl_arr($bus,$darr=array()) { //!$data - 6-element array
   $size=6; // bytes per bus
   if (count($darr)>00) {
     $i2cctl  = fopen('/dev/xi2cctl', 'w');
     for ($i=0; ($i<$size) && ($i<count($darr)); $i++) if ($darr[$i]!=""){
       fseek ($i2cctl, $bus*$size+$i) ;
       fwrite($i2cctl, chr($darr[$i]+0), 1);
     }
     fclose($i2cctl);
   }
   $i2cctl  = fopen('/dev/xi2cctl', 'r');
   fseek ($i2cctl, $bus*$size) ;
   $data = fread($i2cctl, $size);
   fclose($i2cctl);
   for ($i=0;$i<$size; $i++) $darr[$i]=ord($data[$i]);
   return $darr;
} // end of i2c_ctl()



function i2c_send($width,$bus,$a,$d,$raw=0) { //$a<0 - use raw read/write
   $w=($width==16)?2:1;
   $i2c_fn='/dev/xi2c'.($raw?'raw':(($w==2)?'16':'8')).(($bus==0)?'':'_aux');
   $i2c  = fopen($i2c_fn, 'w');
   fseek ($i2c, $w*$a) ;
   if ($w==1)    $res=fwrite($i2c, chr ($d));
   else          $res=fwrite($i2c, chr (floor($d/256)).chr ($d-256*floor($d/256)));
   fclose($i2c);
   return $res;
} // end of i2c_send()

function smbus_send($a,$d) { // d - array
   $i2c_fn='/dev/xi2c8_aux';
   $i2c  = fopen($i2c_fn, 'w');
   fseek ($i2c, $a) ;
   $cmd=chr(count($d));
   foreach ($d as $b) $cmd.=chr($b);
//   var_dump($cmd);
   $res=fwrite($i2c, $cmd);
   fclose($i2c);
   return $res;
} // end of i2c_send()

function i2c_receive($width,$bus,$a,$raw=0) {
   $w=($width==16)?2:1;
   $i2c_fn='/dev/xi2c'.($raw?'raw':(($w==2)?'16':'8')).(($bus==0)?'':'_aux');
   $i2c  = fopen($i2c_fn, 'r');
   fseek ($i2c, $w*$a) ;
   $data = fread($i2c, $w);
   fclose($i2c);
   if (strlen($data)<$w) return -1;
   $v=unpack(($w==1)?'C':'n1',$data);
   return $v[1];
} // end of i2c_receive()

function i2c_setprot ($bus,$slave,$bit,$value) { //!slave is MSB aligned, LSB==0)
   $i2cprot  = fopen("/dev/xi2cenable", 'r+');
   fseek ($i2cprot,($bus*128)+($slave>>1));
   $data=ord(fread($i2cprot,1));
   if ($value) $data |= (1<<$bit);
   else $data &= ~(1<<$bit);
   fseek ($i2cprot,($bus*128)+($slave>>1));
   fwrite($i2cprot, chr ($data));
   fclose($i2cprot);
} // end of i2c_setprot ()

function i2c_getCMOSClock(){
   $i2c  = fopen('/dev/xi2c8_aux', 'r');
   fseek ($i2c, 0x5102) ; //seconds in clock on the 10369 board
   $data = fread($i2c, 7); //sec/min/hours/days/weekdays/century-months/years, BCD
   fclose($i2c);
   if (strlen($data)<7) return "error";
   $v=unpack('C*',$data);
   $d=array("sec"  =>($v[1]& 0xf)+ 10* (($v[1]>>4)& 0x7),
            "min"  =>($v[2]& 0xf)+ 10* (($v[2]>>4)& 0x7),
            "hrs"  =>($v[3]& 0xf)+ 10* (($v[3]>>4)& 0x3),
            "day"  =>($v[4]& 0xf)+ 10* (($v[4]>>4)& 0x3),
            "wday" => $v[5]& 0x7,
            "month"=>($v[6]& 0xf)+ 10* (($v[6]>>4)& 0x1),
            "year" =>($v[7]& 0xf)+ 10* (($v[7]>>4)& 0xf) + (($v[6] & 0x80)?2000:1900)
           );
   exec("date -s ".sprintf("%02d%02d%02d%02d%04d.%02d",$d["month"],$d["day"],$d["hrs"],$d["min"],$d["year"],$d["sec"]),$out,$ret);
   if ($ret==0) {
      elphel_set_fpga_time(time()+0.0);
   }
   return $ret;
} // end of i2c_getCMOSClock()

function i2c_bcd($v) {
  $d=intval($v,10);
  return $d + 6*floor($d/10);
} // end of i2c_bcd($v)

function i2c_setCMOSClock(){
 $ts=time();
 $data=chr(i2c_bcd(gmdate("s",$ts))).
       chr(i2c_bcd(gmdate("i",$ts))).
       chr(i2c_bcd(gmdate("H",$ts))).
       chr(i2c_bcd(gmdate("d",$ts))).
       chr(i2c_bcd(gmdate("w",$ts))).
       chr(i2c_bcd(gmdate("m",$ts))+ ((intval(gmdate("Y",$ts),10)>=2000)?0x80:0)).
       chr(i2c_bcd(gmdate("y",$ts)));
   $i2c  = fopen('/dev/xi2c8_aux', 'w');
   fseek ($i2c, 0x5102) ; //seconds in clock on the 10369 board
   $written= fwrite($i2c, $data);
   fclose($i2c);
   if ($written < strlen($data)) return "error";
   return "OK";
} // end of i2c_setCMOSClock()

function i2c_read256b($slave=0xa0,$bus=1) { //will read 256 bytes from slave (address is 8-bit, includes r/~w)
   global $i2c_slow_safe;
   if ($bus==0) {
     $i2c_old_ctrl= i2c_ctl_arr(0); 
     i2c_ctl_arr(0,$i2c_slow_safe);
   }
   $i2c_fn='/dev/xi2c8'.(($bus==0)?'':'_aux');
   $i2c  = fopen($i2c_fn, 'r');
   fseek ($i2c, $slave*128) ; //256 per slave, but slave are only even
   $data = fread($i2c, 256);  // full 256 bytes
   fclose($i2c);
   if ($bus==0) {
     i2c_ctl_arr(0,$i2c_old_ctrl); /// restore old speed (not thread-safe)
   }
   return $data;
} // end of i2c_read256b ()

//this EEPROM writes only 4 bytes sequentionally (only 2 LSBs are incremented)
function i2c_write256b($data, $slave=0xa0,$bus=1) { //will write up to 256 bytes $data to slave (address is 8-bit, includes r/~w). EEPROM should be un-protected
   global $i2c_slow_safe;
   if ($bus==0) {
     $i2c_old_ctrl= i2c_ctl_arr(0); 
     i2c_ctl_arr(0,$i2c_slow_safe);
   }
   $maxretries=200; //measured - 19
   if (!is_string($data)) return -1;
   if (strlen($data)>256) return -2;
   if (strlen($data)<256) $data.=chr(0);
   $i2c_fn='/dev/xi2c8'.(($bus==0)?'':'_aux');
   $i2c  = fopen($i2c_fn, 'w');
   $len=0;
   for ($i=0;$i<strlen($data); $i+=4) {
     for ($retry=0; $retry< $maxretries; $retry++) {
       fseek ($i2c, $slave*128+$i) ; //256 per slave, but slave are only even
       $rslt=fwrite($i2c, substr($data,$i,4));
       if ($rslt>0) break;
     }
     if ($rslt<=0) {
      $len=$rslt;
      break;
     }
     $len+=$rslt;
   }
   fclose($i2c);
   if ($bus==0) {
     i2c_ctl_arr(0,$i2c_old_ctrl); /// restore old speed (not thread-safe)
   }
   return $len;
} // end of i2c_write256b ()
?>
