<?php
/*!***************************************************************************
*! FILE NAME  : sync_other2this.php
*! DESCRIPTION: synchronizes frame numbers in 2 connected cameras
*!  (should use common external sync)
*! Copyright (C) 2010 Elphel, Inc
*! -----------------------------------------------------------------------------**
*!
*!  This program is free software: you can redistribute it and/or modify
*!  it under the terms of the GNU General Public License as published by
*!  the Free Software Foundation, either version 3 of the License, or
*!  (at your option) any later version.
*!
*!  This program is distributed in the hope that it will be useful,
*!  but WITHOUT ANY WARRANTY; without even the implied warranty of
*!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*!  GNU General Public License for more details.
*!
*!  You should have received a copy of the GNU General Public License
*!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*! -----------------------------------------------------------------------------**
*!  $Log: sync_other2this.php,v $
*!  Revision 1.2  2010/08/14 21:35:42  elphel
*!  added "nocorrect" option (just show frame numbers)
*!
*!  Revision 1.1  2010/08/14 20:44:41  elphel
*!  Added sync_other2this - synchronizing frame numbers in otherwise synchronized camera (eyesis)
*!
*/
$hardware_mask=7; // only 8 frames command buffer in hardware
$fpga_trig_period=0x7b;

/// Synchronization takes place in 2 steps:
/// 1 - synchronizing hardware frame numbers by turning trigger off on the other camera for up to 7 frames
/// 2 - writing software (full) frame number to other camera

if (isset($_GET['frame'])) {
  elphel_set_P_value(ELPHEL_THIS_FRAME,$_GET['frame']+0);
  echo 'OK';
  exit (0);
}
if (isset($_GET['cmd'])) {
  switch ($_GET['cmd']) {
    case 'frame': echo elphel_get_frame(); break;
    case 'stop':
      elphel_fpga_write($fpga_trig_period,0);
      echo 'OK';
      break;
    case 'start':
      elphel_fpga_write($fpga_trig_period,elphel_get_P_value(ELPHEL_TRIG_PERIOD)); // bypassing parameters - everything is stopped
      echo 'OK';
      break;
  }
  exit (0);
}
$ip=$_GET['ip']; if ($ip=='') $ip=$_GET['IP'];
$nocorrect=isset($_GET['nocorrect']);
if ($ip=='') {
   echo <<<USAGE
   <p>This program synchronizes frame number of the other (cable-synchronized) camera to this one</p>
   <p>Required parameter is IP of the of the other camera (?ip=nnn.nnn.nnn.nnn)
USAGE;
  exit (1);
}
   echo <<<HEAD
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
                      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
 <title>Inter-camera synchoronizetion of frame numbers</title>
 <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
</head>
HEAD;
   elphel_skip_frames(); // sync to itself
   $own_frame_number=elphel_get_frame();
   $other_frame_number=file_get_contents('http://'.$ip.'/sync_other2this.php?cmd=frame')+0;
   $skip= (1+$hardware_mask + ($other_frame_number& $hardware_mask)  - ($own_frame_number & $hardware_mask)) & $hardware_mask;
   if ($other_frame_number==$own_frame_number) $nocorrect=true;
   if (!$nocorrect) {
     if ($skip>0) {
        file_get_contents('http://'.$ip.'/sync_other2this.php?cmd=stop');
        elphel_skip_frames($skip); // sync to itself
        file_get_contents('http://'.$ip.'/sync_other2this.php?cmd=start');
     }
     $own_frame_number1=elphel_get_frame();
     if ($own_frame_number1!=$other_frame_number) {
       file_get_contents('http://'.$ip.'/sync_other2this.php?frame='.$own_frame_number1);
     }
     $other_frame_number1=file_get_contents('http://'.$ip.'/sync_other2this.php?cmd=frame')+0;
     $delta1=$other_frame_number1-$own_frame_number1;
   }
   $delta=$other_frame_number-$own_frame_number;
   
   echo <<<BODY1
<body>
<table>
 <tr>
  <td colspan="2">Before syncronization:</td>
 </tr>
 <tr>
  <td>This frame number:</td> <td>$own_frame_number</td>
 </tr>
 <tr>
  <td>$ip frame number:</td> <td>$other_frame_number</td>
 </tr>
 <tr>
  <td>delta:</td> <td>$delta</td>
 </tr>
BODY1;
if (!$nocorrect) {
   echo <<<BODY2
 <tr>
  <td colspan="2">After syncronization:</td>
 </tr>
 <tr>
  <td>This frame number:</td> <td>$own_frame_number1</td>
 </tr>
 <tr>
  <td>$ip frame number:</td> <td>$other_frame_number1</td>
 </tr>
 <tr>
  <td>delta:</td> <td>$delta1</td>
 </tr>
BODY2;
}
   echo <<<BODY3
</table>
</body>
BODY3;


?> 
